@@ -16,7 +16,8 @@
 #include <string.h>
 
 static void
-proxy_toggle_button_toggled_cb (GtkToggleButton* button, GObject* object)
+proxy_toggle_button_toggled_cb (GtkToggleButton* button,
+                                GObject*         object)
 {
     gboolean toggled = gtk_toggle_button_get_active (button);
     const gchar* property = g_object_get_data (G_OBJECT (button), "property");
@@ -50,13 +51,13 @@ proxy_uri_file_set_cb (GtkFileChooser* button,
     g_object_set (object, property, file, NULL);
 }
 
-static gchar*
-proxy_combo_box_text_changed_cb (GtkComboBox* button, GObject* object)
+static void
+proxy_combo_box_text_changed_cb (GtkComboBox* button,
+                                 GObject*     object)
 {
     gchar* text = gtk_combo_box_get_active_text (button);
     const gchar* property = g_object_get_data (G_OBJECT (button), "property");
     g_object_set (object, property, text, NULL);
-    return FALSE;
 }
 
 static void
@@ -79,8 +80,9 @@ proxy_entry_focus_out_event_cb (GtkEntry*      entry,
     return FALSE;
 }
 
-static gboolean
-proxy_spin_button_changed_cb (GtkSpinButton* button, GObject* object)
+static void
+proxy_spin_button_changed_cb (GtkSpinButton* button,
+                              GObject*       object)
 {
     GObjectClass* class = G_OBJECT_GET_CLASS (object);
     const gchar* property = g_object_get_data (G_OBJECT (button), "property");
@@ -95,17 +97,38 @@ proxy_spin_button_changed_cb (GtkSpinButton* button, GObject* object)
         gdouble value = gtk_spin_button_get_value (button);
         g_object_set (object, property, value, NULL);
     }
-    return FALSE;
 }
 
 static void
-proxy_combo_box_changed_cb (GtkComboBox* button, GObject* object)
+proxy_combo_box_changed_cb (GtkComboBox* button,
+                            GObject*     object)
 {
     gint value = gtk_combo_box_get_active (button);
     const gchar* property = g_object_get_data (G_OBJECT (button), "property");
     g_object_set (object, property, value, NULL);
 }
 
+static void
+proxy_object_notify_boolean_cb (GObject*    object,
+                                GParamSpec* pspec,
+                                GtkWidget*  proxy)
+{
+    const gchar* property = g_object_get_data (G_OBJECT (proxy), "property");
+    gboolean value = katze_object_get_boolean (object, property);
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (proxy), value);
+}
+
+static void
+proxy_object_notify_string_cb (GObject*    object,
+                               GParamSpec* pspec,
+                               GtkWidget*  proxy)
+{
+    const gchar* property = g_object_get_data (G_OBJECT (proxy), "property");
+    gchar* value = katze_object_get_string (object, property);
+    gtk_entry_set_text (GTK_ENTRY (proxy), value);
+    g_free (value);
+}
+
 /**
  * katze_property_proxy:
  * @object: a #GObject
@@ -130,6 +153,9 @@ proxy_combo_box_changed_cb (GtkComboBox* button, GObject* object)
  *
  * Any other values for @hint are silently ignored.
  *
+ * Since 0.1.2 strings without hints and booleans are truly synchronous
+ * including property changes causing the proxy to be updated.
+ *
  * Return value: a new widget
  **/
 GtkWidget*
@@ -164,12 +190,17 @@ katze_property_proxy (gpointer     object,
     string = NULL;
     if (type == G_TYPE_PARAM_BOOLEAN)
     {
+        gchar* notify_property;
         gboolean toggled = katze_object_get_boolean (object, property);
 
         widget = gtk_check_button_new_with_label (gettext (nick));
         gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), toggled);
         g_signal_connect (widget, "toggled",
                           G_CALLBACK (proxy_toggle_button_toggled_cb), object);
+        notify_property = g_strdup_printf ("notify::%s", property);
+        g_signal_connect (object, notify_property,
+            G_CALLBACK (proxy_object_notify_boolean_cb), widget);
+        g_free (notify_property);
     }
     else if (type == G_TYPE_PARAM_STRING && _hint == g_intern_string ("file"))
     {
@@ -257,6 +288,8 @@ katze_property_proxy (gpointer     object,
     }
     else if (type == G_TYPE_PARAM_STRING)
     {
+        gchar* notify_property;
+
         widget = gtk_entry_new ();
         g_object_get (object, property, &string, NULL);
         if (!string)
@@ -266,6 +299,10 @@ katze_property_proxy (gpointer     object,
                           G_CALLBACK (proxy_entry_activate_cb), object);
         g_signal_connect (widget, "focus-out-event",
                           G_CALLBACK (proxy_entry_focus_out_event_cb), object);
+        notify_property = g_strdup_printf ("notify::%s", property);
+        g_signal_connect (object, notify_property,
+            G_CALLBACK (proxy_object_notify_string_cb), widget);
+        g_free (notify_property);
     }
     else if (type == G_TYPE_PARAM_FLOAT)
     {
@@ -296,7 +333,7 @@ katze_property_proxy (gpointer     object,
     }
     else if (type == G_TYPE_PARAM_ENUM)
     {
-        gint i;
+        guint i;
         GEnumClass* enum_class = G_ENUM_CLASS (
             g_type_class_ref (pspec->value_type));
         gint value = katze_object_get_enum (object, property);
@@ -316,6 +353,7 @@ katze_property_proxy (gpointer     object,
         widget = gtk_label_new (gettext (nick));
     g_free (string);
 
+    gtk_widget_set_tooltip_text (widget, g_param_spec_get_blurb (pspec));
     gtk_widget_set_sensitive (widget, pspec->flags & G_PARAM_WRITABLE);
 
     g_object_set_data_full (G_OBJECT (widget), "property",
@@ -337,17 +375,23 @@ GtkWidget*
 katze_property_label (gpointer     object,
                       const gchar* property)
 {
+    GObjectClass* class;
+    GParamSpec* pspec;
+    const gchar* nick;
+    GtkWidget* widget;
+
     g_return_val_if_fail (G_IS_OBJECT (object), NULL);
-    GObjectClass* class = G_OBJECT_GET_CLASS (object);
-    GParamSpec* pspec = g_object_class_find_property (class, property);
+
+    class = G_OBJECT_GET_CLASS (object);
+    pspec = g_object_class_find_property (class, property);
     if (!pspec)
     {
         g_warning (_("Property '%s' is invalid for %s"),
                    property, G_OBJECT_CLASS_NAME (class));
         return gtk_label_new (property);
     }
-    const gchar* nick = g_param_spec_get_nick (pspec);
-    GtkWidget* widget = gtk_label_new (nick);
+    nick = g_param_spec_get_nick (pspec);
+    widget = gtk_label_new (nick);
 
     return widget;
 }
@@ -528,10 +572,33 @@ katze_pixbuf_new_from_buffer (const guchar* buffer,
     return pixbuf;
 }
 
+/**
+ * katze_object_has_property:
+ * @object: a #GObject
+ * @property: the name of the property
+ *
+ * Determine if @object has a property with the specified name.
+ *
+ * Return value: a boolean
+ *
+ * Since: 0.1.2
+ **/
+gboolean
+katze_object_has_property (gpointer     object,
+                           const gchar* property)
+{
+    GObjectClass* class;
+
+    g_return_val_if_fail (G_IS_OBJECT (object), FALSE);
+
+    class = G_OBJECT_GET_CLASS (object);
+    return g_object_class_find_property (class, property) != NULL;
+}
+
 /**
  * katze_object_get_boolean:
  * @object: a #GObject
- * @property_name: the name of the property to get
+ * @property: the name of the property to get
  *
  * Retrieve the boolean value of the specified property.
  *
@@ -553,7 +620,7 @@ katze_object_get_boolean (gpointer     object,
 /**
  * katze_object_get_int:
  * @object: a #GObject
- * @property_name: the name of the property to get
+ * @property: the name of the property to get
  *
  * Retrieve the integer value of the specified property.
  *
@@ -575,7 +642,7 @@ katze_object_get_int (gpointer     object,
 /**
  * katze_object_get_float:
  * @object: a #GObject
- * @property_name: the name of the property to get
+ * @property: the name of the property to get
  *
  * Retrieve the float value of the specified property.
  *
@@ -597,7 +664,7 @@ katze_object_get_float (gpointer     object,
 /**
  * katze_object_get_enum:
  * @object: a #GObject
- * @property_name: the name of the property to get
+ * @property: the name of the property to get
  *
  * Retrieve the enum value of the specified property.
  *
@@ -619,7 +686,7 @@ katze_object_get_enum (gpointer     object,
 /**
  * katze_object_get_string:
  * @object: a #GObject
- * @property_name: the name of the property to get
+ * @property: the name of the property to get
  *
  * Retrieve the string value of the specified property.
  *
@@ -641,7 +708,7 @@ katze_object_get_string (gpointer     object,
 /**
  * katze_object_get_object:
  * @object: a #GObject
- * @property_name: the name of the property to get
+ * @property: the name of the property to get
  *
  * Retrieve the object value of the specified property.
  *
