@@ -108,17 +108,11 @@ midori_browser_new_window_cb (MidoriBrowser* browser,
                               const gchar*   uri,
                               MidoriApp*     app)
 {
-    MidoriBrowser* new_browser = g_object_new (MIDORI_TYPE_BROWSER,
-                                               "settings", app->settings,
-                                               "bookmarks", app->bookmarks,
-                                               "trash", app->trash,
-                                               "search-engines", app->search_engines,
-                                               "history", app->history,
-                                               NULL);
+    MidoriBrowser* new_browser = midori_app_create_browser (app);
+    midori_app_add_browser (app, new_browser);
+
     midori_browser_add_uri (new_browser, uri);
     gtk_widget_show (GTK_WIDGET (new_browser));
-
-    g_signal_emit (app, signals[ADD_BROWSER], 0, new_browser);
 }
 
 static gboolean
@@ -147,7 +141,7 @@ midori_browser_quit_cb (MidoriBrowser* browser,
     midori_app_quit (app);
 }
 
-void
+static void
 _midori_app_add_browser (MidoriApp*     app,
                          MidoriBrowser* browser)
 {
@@ -166,11 +160,16 @@ _midori_app_add_browser (MidoriApp*     app,
     app->browsers = g_list_prepend (app->browsers, browser);
 
     #if HAVE_UNIQUE
-    if (app->instance)
-        unique_app_watch_window (app->instance, GTK_WINDOW (browser));
+    unique_app_watch_window (app->instance, GTK_WINDOW (browser));
     #endif
 }
 
+static void
+_midori_app_quit (MidoriApp* app)
+{
+    gtk_main_quit ();
+}
+
 static void
 midori_app_class_init (MidoriAppClass* class)
 {
@@ -203,7 +202,7 @@ midori_app_class_init (MidoriAppClass* class)
     gobject_class->get_property = midori_app_get_property;
 
     class->add_browser = _midori_app_add_browser;
-    class->quit = midori_app_quit;
+    class->quit = _midori_app_quit;
 
     g_object_class_install_property (gobject_class,
                                      PROP_SETTINGS,
@@ -283,7 +282,7 @@ static UniqueResponse
 midori_browser_message_received_cb (UniqueApp*         instance,
                                     UniqueCommand      command,
                                     UniqueMessageData* message,
-                                    guint              time,
+                                    guint              timestamp,
                                     MidoriApp*         app)
 {
   UniqueResponse response;
@@ -301,17 +300,11 @@ midori_browser_message_received_cb (UniqueApp*         instance,
       response = UNIQUE_RESPONSE_OK;
       break;
   case UNIQUE_NEW:
-      browser = g_object_new (MIDORI_TYPE_BROWSER,
-                              "settings", app->settings,
-                              "bookmarks", app->bookmarks,
-                              "trash", app->trash,
-                              "search-engines", app->search_engines,
-                              "history", app->history,
-                              NULL);
+      browser = midori_app_create_browser (app);
+      midori_app_add_browser (app, browser);
       /* FIXME: Should open the homepage according to settings */
       midori_browser_add_uri (browser, "");
       midori_browser_activate_action (browser, "Location");
-      midori_app_add_browser (app, browser);
       gtk_window_set_screen (GTK_WINDOW (app->browser),
                              unique_message_data_get_screen (message));
       gtk_widget_show (GTK_WIDGET (browser));
@@ -330,13 +323,7 @@ midori_browser_message_received_cb (UniqueApp*         instance,
               &open_external_pages_in, NULL);
           if (open_external_pages_in == MIDORI_NEW_PAGE_WINDOW)
           {
-              browser = g_object_new (MIDORI_TYPE_BROWSER,
-                  "settings", app->settings,
-                  "bookmarks", app->bookmarks,
-                  "trash", app->trash,
-                  "search-engines", app->search_engines,
-                  "history", app->history,
-                  NULL);
+              browser = midori_app_create_browser (app);
               midori_app_add_browser (app, browser);
               gtk_window_set_screen (GTK_WINDOW (app->browser),
                                      unique_message_data_get_screen (message));
@@ -381,7 +368,7 @@ midori_app_init (MidoriApp* app)
 
     app->accel_group = gtk_accel_group_new ();
 
-    app->settings = midori_web_settings_new ();
+    app->settings = NULL;
     app->bookmarks = NULL;
     app->trash = NULL;
     app->search_engines = NULL;
@@ -564,7 +551,7 @@ midori_app_instance_send_activate (MidoriApp* app)
     UniqueResponse response;
     #endif
 
-    g_return_val_if_fail (MIDORI_IS_APP (app), FALSE);
+    /* g_return_val_if_fail (MIDORI_IS_APP (app), FALSE); */
     g_return_val_if_fail (midori_app_instance_is_running (app), FALSE);
 
     #if HAVE_UNIQUE
@@ -591,7 +578,7 @@ midori_app_instance_send_new_browser (MidoriApp* app)
     UniqueResponse response;
     #endif
 
-    g_return_val_if_fail (MIDORI_IS_APP (app), FALSE);
+    /* g_return_val_if_fail (MIDORI_IS_APP (app), FALSE); */
     g_return_val_if_fail (midori_app_instance_is_running (app), FALSE);
 
     #if HAVE_UNIQUE
@@ -623,7 +610,7 @@ midori_app_instance_send_uris (MidoriApp* app,
     UniqueResponse response;
     #endif
 
-    g_return_val_if_fail (MIDORI_IS_APP (app), FALSE);
+    /* g_return_val_if_fail (MIDORI_IS_APP (app), FALSE); */
     g_return_val_if_fail (midori_app_instance_is_running (app), FALSE);
     g_return_val_if_fail (uris != NULL, FALSE);
 
@@ -643,7 +630,7 @@ midori_app_instance_send_uris (MidoriApp* app,
  * @app: a #MidoriApp
  * @browser: a #MidoriBrowser
  *
- * Adds a #MidoriBrowser to the #MidoriApp singleton.
+ * Adds a #MidoriBrowser to the #MidoriApp.
  *
  * The app will take care of the browser's new-window and quit signals, as well
  * as watch window closing so that the last closed window quits the app.
@@ -661,16 +648,43 @@ midori_app_add_browser (MidoriApp*     app,
     g_signal_emit (app, signals[ADD_BROWSER], 0, browser);
 }
 
+/**
+ * midori_app_create_browser:
+ * @app: a #MidoriApp
+ *
+ * Creates a #MidoriBrowser which inherits its settings,
+ * bookmarks, trash, search engines and history from  @app.
+ *
+ * Return value: a new #MidoriBrowser
+ *
+ * Since: 1.0.2
+ **/
+MidoriBrowser*
+midori_app_create_browser (MidoriApp* app)
+{
+    g_return_val_if_fail (MIDORI_IS_APP (app), NULL);
+
+    return g_object_new (MIDORI_TYPE_BROWSER,
+                         "settings", app->settings,
+                         "bookmarks", app->bookmarks,
+                         "trash", app->trash,
+                         "search-engines", app->search_engines,
+                         "history", app->history,
+                         NULL);
+}
+
 /**
  * midori_app_quit:
  * @app: a #MidoriApp
  *
  * Quits the #MidoriApp singleton.
+ *
+ * Since 0.1.2 the "quit" signal is always emitted before quitting.
  **/
 void
 midori_app_quit (MidoriApp* app)
 {
     g_return_if_fail (MIDORI_IS_APP (app));
 
-    gtk_main_quit ();
+    g_signal_emit (app, signals[QUIT], 0);
 }
