@@ -13,6 +13,7 @@
 #include "midori-locationaction.h"
 
 #include "gtkiconentry.h"
+#include "marshal.h"
 #include "sokoke.h"
 
 #include <string.h>
@@ -32,6 +33,7 @@ struct _MidoriLocationAction
     GtkTreeModel* filter_model;
     GtkTreeModel* sort_model;
     GdkPixbuf* default_icon;
+    GHashTable* items;
 };
 
 struct _MidoriLocationActionClass
@@ -94,42 +96,6 @@ static void
 midori_location_action_disconnect_proxy (GtkAction* action,
                                          GtkWidget* proxy);
 
-static void
-midori_cclosure_marshal_VOID__STRING_BOOLEAN (GClosure*     closure,
-                                              GValue*       return_value,
-                                              guint         n_param_values,
-                                              const GValue* param_values,
-                                              gpointer      invocation_hint,
-                                              gpointer      marshal_data)
-{
-    typedef void(*GMarshalFunc_VOID__STRING_BOOLEAN) (gpointer      data1,
-                                                      const gchar*  arg_1,
-                                                      gboolean      arg_2,
-                                                      gpointer      data2);
-    register GMarshalFunc_VOID__STRING_BOOLEAN callback;
-    register GCClosure* cc = (GCClosure*) closure;
-    register gpointer data1, data2;
-
-    g_return_if_fail (n_param_values == 3);
-
-    if (G_CCLOSURE_SWAP_DATA (closure))
-    {
-        data1 = closure->data;
-        data2 = g_value_peek_pointer (param_values + 0);
-    }
-    else
-    {
-        data1 = g_value_peek_pointer (param_values + 0);
-        data2 = closure->data;
-    }
-    callback = (GMarshalFunc_VOID__STRING_BOOLEAN) (marshal_data
-        ? marshal_data : cc->callback);
-    callback (data1,
-              g_value_get_string (param_values + 1),
-              g_value_get_boolean (param_values + 2),
-              data2);
-}
-
 static void
 midori_location_action_class_init (MidoriLocationActionClass* class)
 {
@@ -215,29 +181,107 @@ midori_location_action_class_init (MidoriLocationActionClass* class)
 }
 
 static void
-midori_location_action_init (MidoriLocationAction* location_action)
+midori_location_action_set_model (MidoriLocationAction* location_action,
+                                  GtkTreeModel*         model)
+{
+    GSList* proxies;
+    GtkWidget* alignment;
+    GtkWidget* location_entry;
+    GtkWidget* entry;
+
+    proxies = gtk_action_get_proxies (GTK_ACTION (location_action));
+
+    for (; proxies != NULL; proxies = g_slist_next (proxies))
+    if (GTK_IS_TOOL_ITEM (proxies->data))
+    {
+        alignment = gtk_bin_get_child (GTK_BIN (proxies->data));
+        location_entry = gtk_bin_get_child (GTK_BIN (alignment));
+        entry = gtk_bin_get_child (GTK_BIN (location_entry));
+
+        g_object_set (location_entry, "model", model, NULL);
+        g_object_set (gtk_entry_get_completion (GTK_ENTRY (entry)),
+            "model", model ? location_action->sort_model : NULL, NULL);
+    }
+}
+
+static gboolean
+midori_location_action_is_frozen (MidoriLocationAction* location_action)
+{
+    return location_action->filter_model == NULL;
+}
+
+/**
+ * midori_location_action_freeze:
+ * @location_action: a #MidoriLocationAction
+ *
+ * Freezes the action, which essentially means disconnecting models
+ * from proxies and skipping updates of the current state.
+ *
+ * Freezing is recommended if you need to insert a large amount
+ * of items at once, which is faster in the frozen state.
+ *
+ * Use midori_location_action_thaw() to go back to normal operation.
+ *
+ * Since: 0.1.2
+ **/
+void
+midori_location_action_freeze (MidoriLocationAction* location_action)
+{
+    g_return_if_fail (MIDORI_IS_LOCATION_ACTION (location_action));
+    g_return_if_fail (!midori_location_action_is_frozen (location_action));
+
+    katze_object_assign (location_action->sort_model, NULL);
+    katze_object_assign (location_action->filter_model, NULL);
+    midori_location_action_set_model (location_action, NULL);
+}
+
+/**
+ * midori_location_action_thaw:
+ * @location_action: a #MidoriLocationAction
+ *
+ * Thaws the action, ie. after freezing it.
+ *
+ * Since: 0.1.2
+ **/
+void
+midori_location_action_thaw (MidoriLocationAction* location_action)
 {
-    GtkTreeModel* liststore;
     GtkTreeModel* sort_model;
     GtkTreeModel* filter_model;
 
-    location_action->uri = NULL;
-    location_action->progress = 0.0;
+    g_return_if_fail (MIDORI_IS_LOCATION_ACTION (location_action));
+    g_return_if_fail (midori_location_action_is_frozen (location_action));
 
-    liststore = (GtkTreeModel*)gtk_list_store_new (N_COLS,
-        GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING,
-        G_TYPE_INT, G_TYPE_BOOLEAN);
-    sort_model = (GtkTreeModel*)gtk_tree_model_sort_new_with_model (liststore);
+    sort_model = (GtkTreeModel*)gtk_tree_model_sort_new_with_model (
+        location_action->model);
     gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (sort_model),
         VISITS_COL, GTK_SORT_DESCENDING);
     filter_model = gtk_tree_model_filter_new (sort_model, NULL);
     gtk_tree_model_filter_set_visible_column (
         GTK_TREE_MODEL_FILTER (filter_model), VISIBLE_COL);
+    midori_location_action_set_model (location_action, location_action->model);
 
-    location_action->model = liststore;
     location_action->filter_model = filter_model;
     location_action->sort_model = sort_model;
+}
+
+static void
+midori_location_action_init (MidoriLocationAction* location_action)
+{
+    location_action->uri = NULL;
+    location_action->progress = 0.0;
     location_action->default_icon = NULL;
+
+    location_action->model = (GtkTreeModel*)gtk_list_store_new (N_COLS,
+        GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING,
+        G_TYPE_INT, G_TYPE_BOOLEAN);
+
+    location_action->filter_model = NULL;
+    location_action->sort_model = NULL;
+    midori_location_action_thaw (location_action);
+
+    location_action->items = g_hash_table_new_full (g_str_hash, g_str_equal,
+                                                    g_free, g_free);
 }
 
 static void
@@ -252,6 +296,8 @@ midori_location_action_finalize (GObject* object)
     katze_object_assign (location_action->filter_model, NULL);
     katze_object_assign (location_action->default_icon, NULL);
 
+    g_hash_table_destroy (location_action->items);
+
     G_OBJECT_CLASS (midori_location_action_parent_class)->finalize (object);
 }
 
@@ -287,10 +333,8 @@ midori_location_action_activate (GtkAction* action)
     GtkWidget* entry;
 
     proxies = gtk_action_get_proxies (action);
-    if (!proxies)
-        return;
 
-    do
+    for (; proxies != NULL; proxies = g_slist_next (proxies))
     if (GTK_IS_TOOL_ITEM (proxies->data))
     {
         alignment = gtk_bin_get_child (GTK_BIN (proxies->data));
@@ -300,7 +344,6 @@ midori_location_action_activate (GtkAction* action)
         Yet we can't predict which one that is, can we? */
         gtk_widget_grab_focus (entry);
     }
-    while ((proxies = g_slist_next (proxies)));
 
     if (GTK_ACTION_CLASS (midori_location_action_parent_class)->activate)
         GTK_ACTION_CLASS (midori_location_action_parent_class)->activate (action);
@@ -514,193 +557,86 @@ midori_location_action_set_item (MidoriLocationAction*    location_action,
                                  GtkTreeIter*             iter,
                                  MidoriLocationEntryItem* item)
 {
-    GtkTreeModel* model;
     GdkPixbuf* icon;
     GdkPixbuf* new_icon;
 
-    model = location_action->model;
-    gtk_list_store_set (GTK_LIST_STORE (model), iter,
+    gtk_list_store_set (GTK_LIST_STORE (location_action->model), iter,
         URI_COL, item->uri, TITLE_COL, item->title, -1);
 
-    gtk_tree_model_get (model, iter, FAVICON_COL, &icon, -1);
+    gtk_tree_model_get (location_action->model, iter, FAVICON_COL, &icon, -1);
     if (item->favicon)
         new_icon = item->favicon;
-    else if (!icon && !item->favicon)
+    else if (!icon)
         new_icon = location_action->default_icon;
     else
         new_icon = NULL;
     if (new_icon)
-        gtk_list_store_set (GTK_LIST_STORE (model), iter,
+        gtk_list_store_set (GTK_LIST_STORE (location_action->model), iter,
             FAVICON_COL, new_icon, -1);
 }
 
-static gboolean
-midori_location_action_child_iter_to_iter (MidoriLocationAction* location_action,
-                                           GtkTreeIter*          iter,
-                                           GtkTreeIter*          child_iter)
-{
-    GtkTreeModel* filter_model;
-    GtkTreeModel* sort_model;
-    GtkTreeIter sort_iter;
-    GtkTreeIter* temp_iter;
-
-    temp_iter = child_iter;
-
-    filter_model = location_action->filter_model;
-    sort_model = location_action->sort_model;
-    gtk_tree_model_sort_convert_child_iter_to_iter
-        (GTK_TREE_MODEL_SORT (sort_model), &sort_iter, child_iter);
-    temp_iter = &sort_iter;
-
-    return gtk_tree_model_filter_convert_child_iter_to_iter
-        (GTK_TREE_MODEL_FILTER (filter_model), iter, temp_iter);
-}
-
-static void
-midori_location_action_set_active_iter (MidoriLocationAction* location_action,
-                                        GtkTreeIter*          iter)
-{
-    GdkPixbuf* pixbuf;
-    GtkTreeModel* model;
-    GSList* proxies;
-    GtkWidget* alignment;
-    GtkWidget* location_entry;
-    GtkWidget* entry;
-    GtkTreeIter parent_iter;
-
-    model = location_action->model;
-
-    /* The filter iter must be set, not the child iter,
-     * but the row must first be set as visible to
-     * convert to a filter iter without error.
-     */
-    gtk_list_store_set (GTK_LIST_STORE (model), iter, VISIBLE_COL, TRUE, -1);
-
-    proxies = gtk_action_get_proxies (GTK_ACTION (location_action));
-    if (!proxies)
-        return;
-
-    do
-    if (GTK_IS_TOOL_ITEM (proxies->data))
-    {
-        alignment = gtk_bin_get_child (GTK_BIN (proxies->data));
-        location_entry = gtk_bin_get_child (GTK_BIN (alignment));
-        entry = gtk_bin_get_child (GTK_BIN (location_entry));
-
-        if (midori_location_action_child_iter_to_iter (
-            location_action, &parent_iter, iter))
-            gtk_combo_box_set_active_iter (GTK_COMBO_BOX (location_entry),
-                                           &parent_iter);
-
-        /* When setting the active iter (when adding or setting an item)
-         * the icon may have changed, so we must update the entry icon.
-         */
-        gtk_tree_model_get (model, iter, FAVICON_COL, &pixbuf, -1);
-
-        gtk_icon_entry_set_icon_from_pixbuf (GTK_ICON_ENTRY (entry),
-            GTK_ICON_ENTRY_PRIMARY, pixbuf);
-        g_object_unref (pixbuf);
-    }
-    while ((proxies = g_slist_next (proxies)));
-}
-
 /**
- * midori_location_action_item_iter:
+ * midori_location_action_iter_lookup:
  * @location_action: a #MidoriLocationAction
  * @uri: a string
- * @iter: a GtkTreeIter
+ * @iter: a #GtkTreeIter
  *
- * Retrieves the iter of the item matching @uri.
+ * Retrieves the iter of the item matching @uri if it was
+ * inserted with midori_location_action_iter_insert().
  *
  * Return value: %TRUE if @uri was found, %FALSE otherwise
  **/
 static gboolean
-midori_location_action_item_iter (MidoriLocationAction* location_action,
-                                  const gchar*          uri,
-                                  GtkTreeIter*          iter)
+midori_location_action_iter_lookup (MidoriLocationAction* location_action,
+                                    const gchar*          uri,
+                                    GtkTreeIter*          iter)
 {
-    GtkTreeModel* model;
-    gchar* tmpuri;
-    gboolean found;
+    gchar* path;
 
-    g_return_val_if_fail (MIDORI_IS_LOCATION_ACTION (location_action), FALSE);
-    g_return_val_if_fail (uri != NULL, FALSE);
-
-    found = FALSE;
-    model = location_action->model;
-    if (gtk_tree_model_get_iter_first (model, iter))
+    if ((path = g_hash_table_lookup (location_action->items, uri)))
     {
-        tmpuri = NULL;
-        do
-        {
-            gtk_tree_model_get (model, iter, URI_COL, &tmpuri, -1);
-            found = !g_ascii_strcasecmp (uri, tmpuri);
-            katze_assign (tmpuri, NULL);
+        if (gtk_tree_model_get_iter_from_string (location_action->model,
+                                                 iter, path))
+            return TRUE;
 
-            if (found)
-                break;
-        }
-        while (gtk_tree_model_iter_next (model, iter));
+        g_hash_table_remove (location_action->items, uri);
     }
-    return found;
+
+    return FALSE;
 }
 
 /**
- * midori_location_action_reset:
- * @location_action: a #MidoriLocationAction
- *
- * Clears the text in the entry and resets the icon.
- **/
-static void
-midori_location_action_reset (MidoriLocationAction* location_action)
-{
-    GSList* proxies;
-    GtkWidget* alignment;
-    GtkWidget* location_entry;
-    GtkWidget* entry;
-
-    g_return_if_fail (MIDORI_IS_LOCATION_ACTION (location_action));
-
-    proxies = gtk_action_get_proxies (GTK_ACTION (location_action));
-    if (!proxies)
-        return;
-
-    do
-    if (GTK_IS_TOOL_ITEM (proxies->data))
-    {
-        alignment = gtk_bin_get_child (GTK_BIN (proxies->data));
-        location_entry = gtk_bin_get_child (GTK_BIN (alignment));
-        entry = gtk_bin_get_child (GTK_BIN (location_entry));
-
-        gtk_entry_set_text (GTK_ENTRY (entry), "");
-        gtk_icon_entry_set_icon_from_stock (GTK_ICON_ENTRY (entry),
-            GTK_ICON_ENTRY_PRIMARY, GTK_STOCK_FILE);
-    }
-    while ((proxies = g_slist_next (proxies)));
-}
-
-/**
- * midori_location_action_set_item_from_uri:
+ * midori_location_action_iter_insert:
  * @location_action: a #MidoriLocationAction
  * @uri: a string
+ * @iter: a #GtkTreeIter
+ * @position: position to insert the new row
  *
- * Finds the item from the list matching @uri
- * and sets it as the active item.
- * If @uri is not found it clears the active item.
+ * Creates a new row for @uri if it doesn't exist, or sets @iter
+ * to the existing iter for @uri.
+ *
+ * Return value: %TRUE if @uri was found, %FALSE otherwise
  **/
-static void
-midori_location_action_set_item_from_uri (MidoriLocationAction* location_action,
-                                          const gchar*          uri)
+static gboolean
+midori_location_action_iter_insert (MidoriLocationAction* location_action,
+                                    const gchar*          uri,
+                                    GtkTreeIter*          iter,
+                                    gint                  position)
 {
-    GtkTreeIter iter;
+    if (!midori_location_action_iter_lookup (location_action, uri, iter))
+    {
+        GtkTreeModel* model;
+        gchar* path;
 
-    g_return_if_fail (MIDORI_IS_LOCATION_ACTION (location_action));
+        model = location_action->model;
+        gtk_list_store_insert (GTK_LIST_STORE (model), iter, position);
+        path = gtk_tree_model_get_string_from_iter (model,  iter);
+        g_hash_table_insert (location_action->items, g_strdup (uri), path);
 
-    if (midori_location_action_item_iter (location_action, uri, &iter))
-        midori_location_action_set_active_iter (location_action, &iter);
-    else
-        midori_location_action_reset (location_action);
+        return FALSE;
+    }
 
+    return TRUE;
 }
 
 static gboolean
@@ -710,9 +646,10 @@ midori_location_entry_match_selected_cb (GtkEntryCompletion*   completion,
                                          MidoriLocationAction* location_action)
 {
     gchar* uri;
-
     gtk_tree_model_get (model, iter, URI_COL, &uri, -1);
-    midori_location_action_set_item_from_uri (location_action, uri);
+
+    midori_location_action_set_uri (location_action, uri);
+    g_signal_emit (location_action, signals[SUBMIT_URI], 0, uri, FALSE);
     g_free (uri);
 
     return FALSE;
@@ -796,12 +733,16 @@ midori_location_action_connect_proxy (GtkAction* action,
     GTK_ACTION_CLASS (midori_location_action_parent_class)->connect_proxy (
         action, proxy);
 
+    location_action = MIDORI_LOCATION_ACTION (action);
+    katze_object_assign (location_action->default_icon,
+                         gtk_widget_render_icon (proxy, GTK_STOCK_FILE,
+                                                 GTK_ICON_SIZE_MENU, NULL));
+
     if (GTK_IS_TOOL_ITEM (proxy))
     {
         alignment = gtk_bin_get_child (GTK_BIN (proxy));
         entry = gtk_bin_get_child (GTK_BIN (alignment));
 
-        location_action = MIDORI_LOCATION_ACTION (action);
         midori_location_entry_set_progress (MIDORI_LOCATION_ENTRY (entry),
             MIDORI_LOCATION_ACTION (action)->progress);
         gtk_combo_box_set_model (GTK_COMBO_BOX (entry),
@@ -872,17 +813,18 @@ midori_location_action_set_text (MidoriLocationAction* location_action,
     GtkWidget* location_entry;
     GtkWidget* entry;
     GtkTreeIter iter;
-    GtkTreeModel* model;
     GdkPixbuf* icon;
 
-    g_return_if_fail (MIDORI_IS_LOCATION_ACTION (location_action));
-    g_return_if_fail (text != NULL);
-
-    proxies = gtk_action_get_proxies (GTK_ACTION (location_action));
-    if (!proxies)
+    if (!(proxies = gtk_action_get_proxies (GTK_ACTION (location_action))))
         return;
 
-    do
+    if (midori_location_action_iter_lookup (location_action, text, &iter))
+        gtk_tree_model_get (location_action->model,
+                            &iter, FAVICON_COL, &icon, -1);
+    else
+        icon = g_object_ref (location_action->default_icon);
+
+    for (; proxies != NULL; proxies = g_slist_next (proxies))
     if (GTK_IS_TOOL_ITEM (proxies->data))
     {
         alignment = gtk_bin_get_child (GTK_BIN (proxies->data));
@@ -890,19 +832,12 @@ midori_location_action_set_text (MidoriLocationAction* location_action,
         entry = gtk_bin_get_child (GTK_BIN (location_entry));
 
         gtk_entry_set_text (GTK_ENTRY (entry), text);
-        if (midori_location_action_item_iter (location_action, text, &iter))
-        {
-            model = location_action->model;
-            gtk_tree_model_get (model, &iter, FAVICON_COL, &icon, -1);
-            gtk_icon_entry_set_icon_from_pixbuf (GTK_ICON_ENTRY (entry),
-                GTK_ICON_ENTRY_PRIMARY, icon);
-        }
-        /* FIXME: Due to a bug in GtkIconEntry we can't reset the icon
-        else
-            gtk_icon_entry_set_icon_from_stock (GTK_ICON_ENTRY (entry),
-                GTK_ICON_ENTRY_PRIMARY, location_action->default_icon);*/
+        gtk_icon_entry_set_icon_from_pixbuf (GTK_ICON_ENTRY (entry),
+            GTK_ICON_ENTRY_PRIMARY, icon);
     }
-    while ((proxies = g_slist_next (proxies)));
+
+    if (icon)
+        g_object_unref (icon);
 }
 
 void
@@ -927,13 +862,11 @@ midori_location_action_set_icon (MidoriLocationAction* location_action,
     GtkWidget* entry;
 
     g_return_if_fail (MIDORI_IS_LOCATION_ACTION (location_action));
-    g_return_if_fail (icon && GDK_IS_PIXBUF (icon));
+    g_return_if_fail (GDK_IS_PIXBUF (icon));
 
     proxies = gtk_action_get_proxies (GTK_ACTION (location_action));
-    if (!proxies)
-        return;
 
-    do
+    for (; proxies != NULL; proxies = g_slist_next (proxies))
     if (GTK_IS_TOOL_ITEM (proxies->data))
     {
         alignment = gtk_bin_get_child (GTK_BIN (proxies->data));
@@ -943,7 +876,6 @@ midori_location_action_set_icon (MidoriLocationAction* location_action,
         gtk_icon_entry_set_icon_from_pixbuf (GTK_ICON_ENTRY (entry),
             GTK_ICON_ENTRY_PRIMARY, icon);
     }
-    while ((proxies = g_slist_next (proxies)));
 }
 
 /**
@@ -962,31 +894,27 @@ midori_location_action_prepend_item (MidoriLocationAction*    location_action,
     GtkTreeModel* filter_model;
     GtkTreeModel* model;
     GtkTreeIter iter;
-    GtkTreeIter index;
+    GtkTreeIter idx;
     gint n;
     gint visits = 0;
 
-    g_return_if_fail (MIDORI_IS_LOCATION_ACTION (location_action));
-    g_return_if_fail (item->uri != NULL);
-
     filter_model = location_action->filter_model;
     model = location_action->model;
 
-    if (midori_location_action_item_iter (location_action, item->uri, &iter))
+    if (midori_location_action_iter_insert (location_action,
+        item->uri, &iter, 0))
     {
-        gtk_tree_model_get_iter_first (model, &index);
+        gtk_tree_model_get_iter_first (model, &idx);
         gtk_tree_model_get (model, &iter, VISITS_COL, &visits, -1);
-        gtk_list_store_move_before (GTK_LIST_STORE (model), &iter, &index);
+        gtk_list_store_move_before (GTK_LIST_STORE (model), &iter, &idx);
     }
-    else
-        gtk_list_store_prepend (GTK_LIST_STORE (model), &iter);
 
     n = gtk_tree_model_iter_n_children (filter_model, NULL);
     if (n > MAX_ITEMS)
     {
-        gtk_tree_model_iter_nth_child (model, &index, NULL, n - 1);
+        gtk_tree_model_iter_nth_child (model, &idx, NULL, n - 1);
         gtk_list_store_set (GTK_LIST_STORE (model),
-                            &index, VISIBLE_COL, FALSE, -1);
+                            &idx, VISIBLE_COL, FALSE, -1);
     }
 
     /* Only increment the visits when we add the uri */
@@ -1013,23 +941,16 @@ midori_location_action_append_item (MidoriLocationAction*    location_action,
     gint n;
     gint visits = 0;
 
-    g_return_if_fail (MIDORI_IS_LOCATION_ACTION (location_action));
-    g_return_if_fail (item->uri != NULL);
-
     model = location_action->model;
 
-    if (!midori_location_action_item_iter (location_action, item->uri, &iter))
-    {
-        gtk_list_store_append (GTK_LIST_STORE (model), &iter);
-
+    if (!midori_location_action_iter_insert (location_action,
+        item->uri, &iter, G_MAXINT))
         n = gtk_tree_model_iter_n_children (model, NULL);
-        gtk_list_store_set (GTK_LIST_STORE (model), &iter, VISIBLE_COL,
-                            (n <= MAX_ITEMS), -1);
-    }
     else
         gtk_tree_model_get (model, &iter, VISITS_COL, &visits, -1);
 
-    gtk_list_store_set (GTK_LIST_STORE (model), &iter, VISITS_COL, ++visits, -1);
+    gtk_list_store_set (GTK_LIST_STORE (model), &iter,
+                        VISITS_COL, ++visits, VISIBLE_COL, n <= MAX_ITEMS, -1);
     midori_location_action_set_item (location_action, &iter, item);
 }
 
@@ -1042,12 +963,15 @@ midori_location_action_add_uri (MidoriLocationAction* location_action,
     g_return_if_fail (MIDORI_IS_LOCATION_ACTION (location_action));
     g_return_if_fail (uri != NULL);
 
-    katze_assign (location_action->uri, g_strdup (uri));
+    if (midori_location_action_is_frozen (location_action))
+        return;
 
     item.favicon = NULL;
     item.uri = uri;
     item.title = NULL;
     midori_location_action_prepend_item (location_action, &item);
+
+    katze_assign (location_action->uri, g_strdup (uri));
 }
 
 void
@@ -1067,20 +991,21 @@ midori_location_action_add_item (MidoriLocationAction* location_action,
     g_return_if_fail (title != NULL);
     g_return_if_fail (!icon || GDK_IS_PIXBUF (icon));
 
-    katze_assign (location_action->uri, g_strdup (uri));
-
     item.favicon = icon;
     item.uri = uri;
     item.title = title;
     midori_location_action_append_item (location_action, &item);
 
-    proxies = gtk_action_get_proxies (GTK_ACTION (location_action));
-    if (!proxies)
+    if (midori_location_action_is_frozen (location_action))
         return;
 
-    do
+    katze_assign (location_action->uri, g_strdup (uri));
+
+    proxies = gtk_action_get_proxies (GTK_ACTION (location_action));
+
+    for (; proxies != NULL; proxies = g_slist_next (proxies))
     if (GTK_IS_TOOL_ITEM (proxies->data) &&
-        !g_strcmp0 (location_action->uri, uri))
+        !strcmp (location_action->uri, uri))
     {
         alignment = gtk_bin_get_child (GTK_BIN (proxies->data));
         location_entry = gtk_bin_get_child (GTK_BIN (alignment));
@@ -1089,7 +1014,6 @@ midori_location_action_add_item (MidoriLocationAction* location_action,
         gtk_icon_entry_set_icon_from_pixbuf (GTK_ICON_ENTRY (entry),
             GTK_ICON_ENTRY_PRIMARY, icon);
     }
-    while ((proxies = g_slist_next (proxies)));
 }
 
 void
@@ -1113,10 +1037,8 @@ midori_location_action_set_icon_for_uri (MidoriLocationAction* location_action,
     midori_location_action_prepend_item (location_action, &item);
 
     proxies = gtk_action_get_proxies (GTK_ACTION (location_action));
-    if (!proxies)
-        return;
 
-    do
+    for (; proxies != NULL; proxies = g_slist_next (proxies))
     if (GTK_IS_TOOL_ITEM (proxies->data) &&
         !g_strcmp0 (location_action->uri, uri))
     {
@@ -1127,7 +1049,6 @@ midori_location_action_set_icon_for_uri (MidoriLocationAction* location_action,
         gtk_icon_entry_set_icon_from_pixbuf (GTK_ICON_ENTRY (entry),
             GTK_ICON_ENTRY_PRIMARY, icon);
     }
-    while ((proxies = g_slist_next (proxies)));
 }
 
 void
@@ -1168,10 +1089,8 @@ midori_location_action_set_progress (MidoriLocationAction* location_action,
     location_action->progress = CLAMP (progress, 0.0, 1.0);
 
     proxies = gtk_action_get_proxies (GTK_ACTION (location_action));
-    if (!proxies)
-        return;
 
-    do
+    for (; proxies != NULL; proxies = g_slist_next (proxies))
     if (GTK_IS_TOOL_ITEM (proxies->data))
     {
         alignment = gtk_bin_get_child (GTK_BIN (proxies->data));
@@ -1180,7 +1099,6 @@ midori_location_action_set_progress (MidoriLocationAction* location_action,
         midori_location_entry_set_progress (MIDORI_LOCATION_ENTRY (entry),
                                             location_action->progress);
     }
-    while ((proxies = g_slist_next (proxies)));
 }
 
 void
@@ -1197,10 +1115,8 @@ midori_location_action_set_secondary_icon (MidoriLocationAction* location_action
     g_return_if_fail (!stock_id || gtk_stock_lookup (stock_id, &stock_item));
 
     proxies = gtk_action_get_proxies (GTK_ACTION (location_action));
-    if (!proxies)
-        return;
 
-    do
+    for (; proxies != NULL; proxies = g_slist_next (proxies))
     if (GTK_IS_TOOL_ITEM (proxies->data))
     {
         alignment = gtk_bin_get_child (GTK_BIN (proxies->data));
@@ -1214,7 +1130,6 @@ midori_location_action_set_secondary_icon (MidoriLocationAction* location_action
             gtk_icon_entry_set_icon_from_pixbuf (GTK_ICON_ENTRY (child),
                 GTK_ICON_ENTRY_SECONDARY, NULL);
     }
-    while ((proxies = g_slist_next (proxies)));
 }
 
 /**
@@ -1237,7 +1152,7 @@ midori_location_action_delete_item_from_uri (MidoriLocationAction* location_acti
     g_return_if_fail (uri != NULL);
 
     model = location_action->model;
-    if (midori_location_action_item_iter (location_action, uri, &iter))
+    if (midori_location_action_iter_lookup (location_action, uri, &iter))
     {
         gtk_tree_model_get (model, &iter, VISITS_COL, &visits, -1);
         if (visits > 1)
@@ -1251,23 +1166,7 @@ midori_location_action_delete_item_from_uri (MidoriLocationAction* location_acti
 void
 midori_location_action_clear (MidoriLocationAction* location_action)
 {
-    GSList* proxies;
-    GtkWidget* alignment;
-    GtkWidget* entry;
-
     g_return_if_fail (MIDORI_IS_LOCATION_ACTION (location_action));
 
-    proxies = gtk_action_get_proxies (GTK_ACTION (location_action));
-    if (!proxies)
-        return;
-
-    do
-    if (GTK_IS_TOOL_ITEM (proxies->data))
-    {
-        alignment = gtk_bin_get_child (GTK_BIN (proxies->data));
-        entry = gtk_bin_get_child (GTK_BIN (alignment));
-
-        gtk_list_store_clear (GTK_LIST_STORE (location_action->filter_model));
-    }
-    while ((proxies = g_slist_next (proxies)));
+    gtk_list_store_clear (GTK_LIST_STORE (location_action->model));
 }
