@@ -27,48 +27,15 @@ typedef struct _MidoriLocationEntry         MidoriLocationEntry;
 typedef struct _MidoriLocationEntryClass    MidoriLocationEntryClass;
 typedef struct _MidoriLocationEntryItem     MidoriLocationEntryItem;
 
-struct _MidoriLocationEntryItem
-{
-    GdkPixbuf* favicon;
-    const gchar* uri;
-    const gchar* title;
-};
-
 GType
-midori_location_entry_get_type (void);
+midori_location_entry_get_type             (void);
 
 GtkWidget*
-midori_location_entry_new (void);
-
-gboolean
-midori_location_entry_item_iter         (MidoriLocationEntry* location_entry,
-                                         const gchar*         uri,
-                                         GtkTreeIter*         iter);
-
-const gchar*
-midori_location_entry_get_text          (MidoriLocationEntry* location_entry);
+midori_location_entry_new                  (void);
 
 void
-midori_location_entry_set_text          (MidoriLocationEntry* location_entry,
-                                         const gchar*         text);
-
-void
-midori_location_entry_clear             (MidoriLocationEntry* location_entry);
-
-void
-midori_location_entry_set_item_from_uri (MidoriLocationEntry* location_entry,
-                                         const gchar*         uri);
-
-void
-midori_location_entry_add_item          (MidoriLocationEntry* location_entry,
-                                         MidoriLocationEntryItem* item);
-
-gdouble
-midori_location_entry_get_progress      (MidoriLocationEntry* location_entry);
-
-void
-midori_location_entry_set_progress      (MidoriLocationEntry* location_entry,
-                                         gdouble              progress);
+midori_location_entry_set_progress         (MidoriLocationEntry* location_entry,
+                                            gdouble              progress);
 
 G_END_DECLS
 
