@@ -12,8 +12,10 @@
 #include "midori-panel.h"
 
 #include "midori-view.h"
-#include "sokoke.h"
+
 #include "compat.h"
+#include "marshal.h"
+#include "sokoke.h"
 
 #include <glib/gi18n.h>
 
@@ -81,40 +83,6 @@ midori_panel_close (MidoriPanel* panel)
     return FALSE;
 }
 
-static void
-midori_cclosure_marshal_BOOLEAN__VOID (GClosure*     closure,
-                                       GValue*       return_value,
-                                       guint         n_param_values,
-                                       const GValue* param_values,
-                                       gpointer      invocation_hint,
-                                       gpointer      marshal_data)
-{
-    typedef gboolean(*GMarshalFunc_BOOLEAN__VOID) (gpointer  data1,
-                                                   gpointer  data2);
-    register GMarshalFunc_BOOLEAN__VOID callback;
-    register GCClosure* cc = (GCClosure*) closure;
-    register gpointer data1, data2;
-    gboolean v_return;
-
-    g_return_if_fail (return_value != NULL);
-    g_return_if_fail (n_param_values == 1);
-
-    if (G_CCLOSURE_SWAP_DATA (closure))
-    {
-        data1 = closure->data;
-        data2 = g_value_peek_pointer (param_values + 0);
-    }
-    else
-    {
-        data1 = g_value_peek_pointer (param_values + 0);
-        data2 = closure->data;
-    }
-    callback = (GMarshalFunc_BOOLEAN__VOID) (marshal_data
-        ? marshal_data : cc->callback);
-    v_return = callback (data1, data2);
-    g_value_set_boolean (return_value, v_return);
-}
-
 static void
 midori_panel_class_init (MidoriPanelClass* class)
 {
@@ -627,6 +595,7 @@ typedef struct
     GtkAlignmentClass parent_class;
 }  MidoriDummyViewableClass;
 
+GType midori_dummy_viewable_get_type (void) G_GNUC_CONST;
 #define MIDORI_TYPE_DUMMY_VIEWABLE (midori_dummy_viewable_get_type ())
 #define MIDORI_DUMMY_VIEWABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), \
     MIDORI_TYPE_DUMMY_VIEWABLE, MidoriDummyViewable))
