@@ -20,6 +20,10 @@
 #include <string.h>
 #include <glib/gi18n.h>
 
+#if HAVE_LIBSOUP
+    #include <libsoup/soup.h>
+#endif
+
 struct _MidoriPreferences
 {
     GtkDialog parent_instance;
@@ -91,7 +95,8 @@ midori_preferences_init (MidoriPreferences* preferences)
                   "title", dialog_title,
                   "has-separator", FALSE,
                   NULL);
-    #ifndef HAVE_OSX
+    g_free (dialog_title);
+    #if !HAVE_OSX
     gtk_dialog_add_buttons (GTK_DIALOG (preferences),
         GTK_STOCK_HELP, GTK_RESPONSE_HELP,
         GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
@@ -99,9 +104,6 @@ midori_preferences_init (MidoriPreferences* preferences)
     #endif
     g_signal_connect (preferences, "response",
                       G_CALLBACK (midori_preferences_response_cb), NULL);
-
-    /* TODO: Do we want tooltips for explanations or can we omit that? */
-    g_free (dialog_title);
 }
 
 static void
@@ -122,14 +124,6 @@ midori_preferences_set_property (GObject*      object,
     {
     case PROP_SETTINGS:
     {
-        GtkWidget* xfce_heading;
-        GtkWindow* parent;
-        g_object_get (object, "transient-for", &parent, NULL);
-        if ((xfce_heading = sokoke_xfce_header_new (
-            gtk_window_get_icon_name (parent),
-            gtk_window_get_title (GTK_WINDOW (object)))))
-                gtk_box_pack_start (GTK_BOX (GTK_DIALOG (preferences)->vbox),
-                                    xfce_heading, FALSE, FALSE, 0);
         midori_preferences_set_settings (preferences,
                                          g_value_get_object (value));
         break;
@@ -161,6 +155,8 @@ midori_preferences_get_property (GObject*    object,
  *
  * Creates a new preferences dialog.
  *
+ * Since 0.1.2 @parent may be %NULL.
+ *
  * Return value: a new #MidoriPreferences
  **/
 GtkWidget*
@@ -169,7 +165,7 @@ midori_preferences_new (GtkWindow*         parent,
 {
     MidoriPreferences* preferences;
 
-    g_return_val_if_fail (GTK_IS_WINDOW (parent), NULL);
+    g_return_val_if_fail (!parent || GTK_IS_WINDOW (parent), NULL);
     g_return_val_if_fail (MIDORI_IS_WEB_SETTINGS (settings), NULL);
 
     preferences = g_object_new (MIDORI_TYPE_PREFERENCES,
@@ -216,7 +212,30 @@ proxy_download_manager_icon_cb (GtkWidget*     entry,
     return FALSE;
 }
 
-#ifdef HAVE_OSX
+static void
+midori_preferences_notify_preferred_encoding_cb (MidoriWebSettings* settings,
+                                                 GParamSpec*        pspec,
+                                                 GtkWidget*         entry)
+{
+    MidoriPreferredEncoding preferred_encoding;
+
+    preferred_encoding = katze_object_get_enum (settings, "preferred-encoding");
+    gtk_widget_set_sensitive (entry, preferred_encoding == MIDORI_ENCODING_CUSTOM);
+}
+
+#if HAVE_LIBSOUP
+static void
+midori_preferences_notify_identify_as_cb (MidoriWebSettings* settings,
+                                          GParamSpec*        pspec,
+                                          GtkWidget*         entry)
+{
+    MidoriIdentity identify_as = katze_object_get_enum (settings, "identify-as");
+
+    gtk_widget_set_sensitive (entry, identify_as == MIDORI_IDENT_CUSTOM);
+}
+#endif
+
+#if HAVE_OSX
 static void
 midori_preferences_help_clicked_cb (GtkWidget* button,
                                     GtkDialog* dialog)
@@ -241,7 +260,7 @@ midori_preferences_add_toolbutton (GtkWidget*   toolbar,
                                    const gchar* label,
                                    GtkWidget*   page)
 {
-#ifdef HAVE_OSX
+#if HAVE_OSX
     *toolbutton = GTK_WIDGET (*toolbutton ? gtk_radio_tool_button_new_from_widget (
         GTK_RADIO_TOOL_BUTTON (*toolbutton)) : gtk_radio_tool_button_new (NULL));
     gtk_tool_button_set_label (GTK_TOOL_BUTTON (*toolbutton), label);
@@ -259,11 +278,16 @@ midori_preferences_add_toolbutton (GtkWidget*   toolbar,
  * Assigns a settings instance to a preferences dialog.
  *
  * Note: This must not be called more than once.
+ *
+ * Since 0.1.2 this is equal to setting #MidoriPreferences:settings:.
  **/
 void
 midori_preferences_set_settings (MidoriPreferences* preferences,
                                  MidoriWebSettings* settings)
 {
+    GtkWidget* header;
+    GtkWindow* parent;
+    const gchar* icon_name;
     GtkSizeGroup* sizegroup;
     GtkWidget* toolbar;
     GtkWidget* toolbutton;
@@ -282,10 +306,17 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
 
     g_return_if_fail (!preferences->notebook);
 
+    g_object_get (preferences, "transient-for", &parent, NULL);
+    icon_name = parent ? gtk_window_get_icon_name (parent) : NULL;
+    if ((header = sokoke_xfce_header_new (icon_name,
+        gtk_window_get_title (GTK_WINDOW (preferences)))))
+        gtk_box_pack_start (GTK_BOX (GTK_DIALOG (preferences)->vbox),
+            header, FALSE, FALSE, 0);
+
     preferences->notebook = gtk_notebook_new ();
     gtk_container_set_border_width (GTK_CONTAINER (preferences->notebook), 6);
 
-    #ifdef HAVE_OSX
+    #if HAVE_OSX
     gtk_notebook_set_show_tabs (GTK_NOTEBOOK (preferences->notebook), FALSE);
     gtk_notebook_set_show_border (GTK_NOTEBOOK (preferences->notebook), FALSE);
     toolbar = gtk_toolbar_new ();
@@ -337,7 +368,7 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     /* Page "General" */
     PAGE_NEW (GTK_STOCK_HOME, _("General"));
     FRAME_NEW (_("Startup"));
-    TABLE_NEW (2, 2);
+    TABLE_NEW (3, 2);
     label = katze_property_label (settings, "load-on-startup");
     INDENTED_ADD (label, 0, 1, 0, 1);
     button = katze_property_proxy (settings, "load-on-startup", NULL);
@@ -347,6 +378,8 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     /* TODO: We need something like "use current website" */
     entry = katze_property_proxy (settings, "homepage", NULL);
     FILLED_ADD (entry, 1, 2, 1, 2);
+    button = katze_property_proxy (settings, "show-crash-dialog", "blurb");
+    SPANNED_ADD (button, 0, 2, 2, 3);
     FRAME_NEW (_("Transfers"));
     TABLE_NEW (3, 2);
     label = katze_property_label (settings, "download-folder");
@@ -385,23 +418,34 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     /* Page "Appearance" */
     PAGE_NEW (GTK_STOCK_SELECT_FONT, _("Appearance"));
     FRAME_NEW (_("Font settings"));
-    TABLE_NEW (5, 2);
+    TABLE_NEW (6, 2);
     label = gtk_label_new (_("Default Font Family"));
     INDENTED_ADD (label, 0, 1, 0, 1);
     hbox = gtk_hbox_new (FALSE, 4);
     button = katze_property_proxy (settings, "default-font-family", "font");
     gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
     entry = katze_property_proxy (settings, "default-font-size", NULL);
+    gtk_widget_set_tooltip_text (entry, _("The default font size used to display text"));
     gtk_box_pack_end (GTK_BOX (hbox), entry, FALSE, FALSE, 4);
     FILLED_ADD (hbox, 1, 2, 0, 1);
     label = gtk_label_new (_("Minimum Font Size"));
     INDENTED_ADD (label, 0, 1, 1, 2);
     entry = katze_property_proxy (settings, "minimum-font-size", NULL);
+    gtk_widget_set_tooltip_text (entry, _("The minimum font size used to display text"));
     INDENTED_ADD (entry, 1, 2, 1, 2);
     label = katze_property_label (settings, "preferred-encoding");
     INDENTED_ADD (label, 0, 1, 2, 3);
     button = katze_property_proxy (settings, "preferred-encoding", NULL);
     FILLED_ADD (button, 1, 2, 2, 3);
+    label = katze_property_label (settings, "default-encoding");
+    gtk_label_set_label (GTK_LABEL (label), _("Encoding"));
+    INDENTED_ADD (label, 0, 1, 3, 4);
+    entry = katze_property_proxy (settings, "default-encoding", NULL);
+    gtk_widget_set_tooltip_text (entry, _("The character encoding to use by default"));
+    g_signal_connect (settings, "notify::preferred-encoding",
+        G_CALLBACK (midori_preferences_notify_preferred_encoding_cb), entry);
+    midori_preferences_notify_preferred_encoding_cb (settings, NULL, entry);
+    FILLED_ADD (entry, 1, 2, 3, 4);
 
     /* Page "Behavior" */
     PAGE_NEW (GTK_STOCK_SELECT_COLOR, _("Behavior"));
@@ -409,29 +453,36 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     TABLE_NEW (6, 2);
     button = katze_property_proxy (settings, "auto-load-images", NULL);
     gtk_button_set_label (GTK_BUTTON (button), _("Load images automatically"));
+    gtk_widget_set_tooltip_text (button, _("Load and display images automatically"));
     INDENTED_ADD (button, 0, 1, 0, 1);
     button = katze_property_proxy (settings, "auto-shrink-images", NULL);
     gtk_button_set_label (GTK_BUTTON (button), _("Shrink images automatically"));
+    gtk_widget_set_tooltip_text (button, _("Automatically shrink standalone images to fit"));
     SPANNED_ADD (button, 1, 2, 0, 1);
     button = katze_property_proxy (settings, "print-backgrounds", NULL);
     gtk_button_set_label (GTK_BUTTON (button), _("Print background images"));
+    gtk_widget_set_tooltip_text (button, _("Whether background images should be printed"));
     INDENTED_ADD (button, 0, 1, 1, 2);
     button = katze_property_proxy (settings, "resizable-text-areas", NULL);
     gtk_button_set_label (GTK_BUTTON (button), _("Resizable text areas"));
+    gtk_widget_set_tooltip_text (button, _("Whether text areas are resizable"));
     SPANNED_ADD (button, 1, 2, 1, 2);
     button = katze_property_proxy (settings, "enable-scripts", NULL);
     gtk_button_set_label (GTK_BUTTON (button), _("Enable scripts"));
+    gtk_widget_set_tooltip_text (button, _("Enable embedded scripting languages"));
     INDENTED_ADD (button, 0, 1, 2, 3);
     button = katze_property_proxy (settings, "enable-plugins", NULL);
     gtk_button_set_label (GTK_BUTTON (button), _("Enable plugins"));
+    gtk_widget_set_tooltip_text (button, _("Enable embedded plugin objects"));
     SPANNED_ADD (button, 1, 2, 2, 3);
-    #ifdef WEBKIT_CHECK_VERSION
-    #if WEBKIT_CHECK_VERSION (1, 0, 3)
+    button = katze_property_proxy (settings, "enforce-96-dpi", NULL);
+    gtk_button_set_label (GTK_BUTTON (button), _("Enforce 96 dots per inch"));
+    gtk_widget_set_tooltip_text (button, _("Enforce a video dot density of 96 DPI"));
+    SPANNED_ADD (button, 0, 1, 3, 4);
     button = katze_property_proxy (settings, "enable-developer-extras", NULL);
     gtk_button_set_label (GTK_BUTTON (button), _("Enable developer tools"));
-    SPANNED_ADD (button, 0, 1, 3, 4);
-    #endif
-    #endif
+    gtk_widget_set_tooltip_text (button, _("Enable special extensions for developers"));
+    SPANNED_ADD (button, 1, 2, 3, 4);
     label = katze_property_label (settings, "location-entry-search");
     INDENTED_ADD (label, 0, 1, 4, 5);
     entry = katze_property_proxy (settings, "location-entry-search", NULL);
@@ -450,57 +501,78 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     button = gtk_label_new (NULL);
     gtk_label_set_markup (GTK_LABEL (button),
         "<span size='smaller'>"
-        "TabNew, Open, Print, Find, Location, Preferences, Window, Bookmarks\n"
+        "Separator, TabNew, Open, Print, Find, Location, Preferences, Window, Bookmarks\n"
         "ReloadStop, ZoomIn, ZoomOut, Back, Forward, Homepage, Search, Trash"
         "</span>");
+    gtk_label_set_selectable (GTK_LABEL (button), TRUE);
     FILLED_ADD (button, 0, 2, 2, 3);
     FRAME_NEW (_("Browsing"));
-    TABLE_NEW (4, 2);
-    label = katze_property_label (settings, "open-external-pages-in");
-    INDENTED_ADD (label, 0, 1, 0, 1);
-    button = katze_property_proxy (settings, "open-external-pages-in", NULL);
-    FILLED_ADD (button, 1, 2, 0, 1);
-    /* label = katze_property_label (settings, "open-new-pages-in");
+    TABLE_NEW (5, 2);
+    label = katze_property_label (settings, "open-new-pages-in");
     INDENTED_ADD (label, 0, 1, 0, 1);
     button = katze_property_proxy (settings, "open-new-pages-in", NULL);
-    FILLED_ADD (button, 1, 2, 0, 1); */
+    FILLED_ADD (button, 1, 2, 0, 1);
+    label = katze_property_label (settings, "open-external-pages-in");
+    INDENTED_ADD (label, 0, 1, 1, 2);
+    button = katze_property_proxy (settings, "open-external-pages-in", NULL);
+    FILLED_ADD (button, 1, 2, 1, 2);
     button = katze_property_proxy (settings, "always-show-tabbar", NULL);
-    INDENTED_ADD (button, 0, 1, 1, 2);
-    button = katze_property_proxy (settings, "compact-sidepanel", NULL);
-    INDENTED_ADD (button, 1, 2, 1, 2);
-    button = katze_property_proxy (settings, "middle-click-opens-selection", NULL);
     INDENTED_ADD (button, 0, 1, 2, 3);
+    button = katze_property_proxy (settings, "compact-sidepanel", NULL);
+    INDENTED_ADD (button, 1, 2, 2, 3);
+    button = katze_property_proxy (settings, "middle-click-opens-selection", NULL);
+    INDENTED_ADD (button, 0, 1, 3, 4);
     button = katze_property_proxy (settings, "open-tabs-in-the-background", NULL);
-    WIDGET_ADD (button, 1, 2, 2, 3);
-    /* button = katze_property_proxy (settings, "open-popups-in-tabs", NULL);
-    SPANNED_ADD (button, 0, 1, 2, 3);*/
-    button = katze_property_proxy (settings, "open-tabs-next-to-current", NULL);
-    WIDGET_ADD (button, 0, 1, 3, 4);
-    button = katze_property_proxy (settings, "close-buttons-on-tabs", NULL);
     WIDGET_ADD (button, 1, 2, 3, 4);
+    /* button = katze_property_proxy (settings, "open-popups-in-tabs", NULL);
+    SPANNED_ADD (button, 0, 1, 4, 5);*/
+    button = katze_property_proxy (settings, "open-tabs-next-to-current", NULL);
+    WIDGET_ADD (button, 0, 1, 5, 6);
+    button = katze_property_proxy (settings, "close-buttons-on-tabs", NULL);
+    WIDGET_ADD (button, 1, 2, 5, 6);
 
     /* Page "Network" */
-    #if 0
+    #if HAVE_LIBSOUP
+    /* If a cookie jar was created, WebKit is using Soup */
+    if (g_type_get_qdata (SOUP_TYPE_COOKIE_JAR,
+        g_quark_from_static_string ("midori-has-jar")))
+    {
     PAGE_NEW (GTK_STOCK_NETWORK, _("Network"));
     FRAME_NEW (_("Network"));
-    TABLE_NEW (2, 2);
+    TABLE_NEW (4, 2);
     label = katze_property_label (settings, "http-proxy");
     INDENTED_ADD (label, 0, 1, 0, 1);
     button = katze_property_proxy (settings, "http-proxy", NULL);
     FILLED_ADD (button, 1, 2, 0, 1);
-    label = katze_property_label (settings, "cache-size");
+    label = katze_property_label (settings, "identify-as");
     INDENTED_ADD (label, 0, 1, 1, 2);
+    button = katze_property_proxy (settings, "identify-as", NULL);
+    FILLED_ADD (button, 1, 2, 1, 2);
+    label = katze_property_label (settings, "ident-string");
+    INDENTED_ADD (label, 0, 1, 2, 3);
+    entry = katze_property_proxy (settings, "ident-string", NULL);
+    g_signal_connect (settings, "notify::identify-as",
+        G_CALLBACK (midori_preferences_notify_identify_as_cb), entry);
+    midori_preferences_notify_identify_as_cb (settings, NULL, entry);
+    FILLED_ADD (entry, 1, 2, 2, 3);
+    label = katze_property_label (settings, "cache-size");
+    INDENTED_ADD (label, 0, 1, 3, 4);
     hbox = gtk_hbox_new (FALSE, 4);
     entry = katze_property_proxy (settings, "cache-size", NULL);
     gtk_box_pack_start (GTK_BOX (hbox), entry, FALSE, FALSE, 0);
     gtk_box_pack_start (GTK_BOX (hbox), gtk_label_new (_("MB")),
                         FALSE, FALSE, 0);
-    FILLED_ADD (hbox, 1, 2, 1, 2);
+    FILLED_ADD (hbox, 1, 2, 3, 4);
+    }
     #endif
 
     /* Page "Privacy" */
     PAGE_NEW (GTK_STOCK_INDEX, _("Privacy"));
-    #if 0
+    #if HAVE_LIBSOUP_2_25_2
+    /* If a cookie jar was created, WebKit is using Soup */
+    if (g_type_get_qdata (SOUP_TYPE_COOKIE_JAR,
+        g_quark_from_static_string ("midori-has-jar")))
+    {
     FRAME_NEW (_("Web Cookies"));
     TABLE_NEW (3, 2);
     label = katze_property_label (settings, "accept-cookies");
@@ -517,6 +589,7 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     gtk_box_pack_start (GTK_BOX (hbox), gtk_label_new (_("days")),
                         FALSE, FALSE, 0);
     FILLED_ADD (hbox, 1, 2, 2, 3);
+    }
     #endif
     FRAME_NEW (_("History"));
     TABLE_NEW (3, 2);
@@ -536,7 +609,7 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     g_object_unref (sizegroup);
     gtk_box_pack_start (GTK_BOX (GTK_DIALOG (preferences)->vbox),
                         preferences->notebook, FALSE, FALSE, 4);
-    #ifdef HAVE_OSX
+    #if HAVE_OSX
     GtkWidget* icon;
     hbox = gtk_hbox_new (FALSE, 0);
     button = gtk_button_new ();
