@@ -12,8 +12,10 @@
 #include "midori-searchaction.h"
 
 #include "gtkiconentry.h"
+#include "marshal.h"
 #include "sokoke.h"
 
+#include <string.h>
 #include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
 
@@ -90,42 +92,6 @@ static void
 midori_search_action_disconnect_proxy (GtkAction* action,
                                        GtkWidget* proxy);
 
-static void
-midori_cclosure_marshal_VOID__STRING_BOOLEAN (GClosure*     closure,
-                                              GValue*       return_value,
-                                              guint         n_param_values,
-                                              const GValue* param_values,
-                                              gpointer      invocation_hint,
-                                              gpointer      marshal_data)
-{
-    typedef void(*GMarshalFunc_VOID__STRING_BOOLEAN) (gpointer      data1,
-                                                      const gchar*  arg_1,
-                                                      gboolean      arg_2,
-                                                      gpointer      data2);
-    register GMarshalFunc_VOID__STRING_BOOLEAN callback;
-    register GCClosure* cc = (GCClosure*) closure;
-    register gpointer data1, data2;
-
-    g_return_if_fail (n_param_values == 3);
-
-    if (G_CCLOSURE_SWAP_DATA (closure))
-    {
-        data1 = closure->data;
-        data2 = g_value_peek_pointer (param_values + 0);
-    }
-    else
-    {
-        data1 = g_value_peek_pointer (param_values + 0);
-        data2 = closure->data;
-    }
-    callback = (GMarshalFunc_VOID__STRING_BOOLEAN) (marshal_data
-        ? marshal_data : cc->callback);
-    callback (data1,
-              g_value_get_string (param_values + 1),
-              g_value_get_boolean (param_values + 2),
-              data2);
-}
-
 static void
 midori_search_action_class_init (MidoriSearchActionClass* class)
 {
@@ -411,6 +377,37 @@ midori_search_action_manage_activate_cb (GtkWidget*          menuitem,
         gtk_widget_show (dialog);
 }
 
+static GdkPixbuf*
+midori_search_action_get_icon (MidoriSearchAction* search_action,
+                               KatzeItem*          item,
+                               GtkWidget*          widget)
+{
+    const gchar* icon;
+
+    if ((icon = katze_item_get_icon (item)) && *icon)
+    {
+        GdkScreen* screen;
+        GtkIconTheme* icon_theme;
+        gint width, height;
+
+        if (G_UNLIKELY (!(screen = gtk_widget_get_screen (widget))))
+            return gtk_widget_render_icon (widget, GTK_STOCK_FILE,
+                                           GTK_ICON_SIZE_MENU, NULL);
+        icon_theme = gtk_icon_theme_get_for_screen (screen);
+        gtk_icon_size_lookup_for_settings (gtk_widget_get_settings (widget),
+            GTK_ICON_SIZE_MENU, &width, &height);
+        return gtk_icon_theme_load_icon (icon_theme, icon, MAX (width, height),
+                                         GTK_ICON_LOOKUP_USE_BUILTIN, NULL);
+    }
+
+    if ((icon = katze_item_get_uri (item)) && strstr (icon, "://"))
+        return katze_net_load_icon (search_action->net,
+            icon, NULL, widget, NULL);
+
+    return gtk_widget_render_icon (widget, GTK_STOCK_FILE,
+                                   GTK_ICON_SIZE_MENU, NULL);
+}
+
 static void
 midori_search_action_icon_released_cb (GtkWidget*           entry,
                                        GtkIconEntryPosition icon_pos,
@@ -439,8 +436,8 @@ midori_search_action_icon_released_cb (GtkWidget*           entry,
             menuitem = gtk_image_menu_item_new_with_label (
                 katze_item_get_name (item));
             image = gtk_image_new ();
-            icon = katze_net_load_icon (MIDORI_SEARCH_ACTION (action)->net,
-                katze_item_get_uri (item), NULL, entry, NULL);
+            icon = midori_search_action_get_icon (MIDORI_SEARCH_ACTION (action),
+                                                  item, entry);
             gtk_image_set_from_pixbuf (GTK_IMAGE (image), icon);
             g_object_unref (icon);
             gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menuitem), image);
@@ -493,9 +490,8 @@ midori_search_action_set_entry_icon (MidoriSearchAction* search_action,
 
     if (search_action->current_item)
     {
-        icon = katze_net_load_icon (search_action->net,
-            katze_item_get_uri (search_action->current_item),
-            NULL, entry, NULL);
+        icon = midori_search_action_get_icon (search_action,
+            search_action->current_item, entry);
         gtk_icon_entry_set_icon_from_pixbuf (GTK_ICON_ENTRY (entry),
                                              GTK_ICON_ENTRY_PRIMARY, icon);
         g_object_unref (icon);
@@ -723,19 +719,16 @@ midori_search_action_dialog_render_icon_cb (GtkTreeViewColumn* column,
                                             GtkTreeIter*       iter,
                                             GtkWidget*         treeview)
 {
-    KatzeNet* net;
     KatzeItem* item;
+    MidoriSearchAction* search_action;
     GdkPixbuf* icon;
 
     gtk_tree_model_get (model, iter, 0, &item, -1);
 
-    /* FIXME: Use the net of the MidoriSearchAction */
-    net = katze_net_new ();
-    icon = katze_net_load_icon (net, katze_item_get_uri (item),
-        NULL, treeview, NULL);
+    search_action = g_object_get_data (G_OBJECT (treeview), "search-action");
+    icon = midori_search_action_get_icon (search_action, item, treeview);
     g_object_set (renderer, "pixbuf", icon, NULL);
     g_object_unref (icon);
-    g_object_unref (net);
 }
 
 static void
@@ -852,7 +845,7 @@ midori_search_action_get_editor (MidoriSearchAction* search_action,
 
     hbox = gtk_hbox_new (FALSE, 8);
     gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);
-    label = gtk_label_new_with_mnemonic (_("_URL:"));
+    label = gtk_label_new_with_mnemonic (_("_Address:"));
     gtk_size_group_add_widget (sizegroup, label);
     gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
     entry_uri = gtk_entry_new ();
@@ -1057,12 +1050,12 @@ midori_search_action_get_dialog (MidoriSearchAction* search_action)
     if (search_action->dialog)
         return search_action->dialog;
 
-    dialog_title = _("Manage search engines");
+    dialog_title = _("Manage Search Engines");
     toplevel = gtk_widget_get_toplevel (search_action->last_proxy);
     dialog = gtk_dialog_new_with_buttons (dialog_title,
         toplevel ? GTK_WINDOW (toplevel) : NULL,
         GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR,
-        #ifndef HAVE_OSX
+        #if !HAVE_OSX
         GTK_STOCK_HELP, GTK_RESPONSE_HELP,
         GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
         #endif
@@ -1096,6 +1089,7 @@ midori_search_action_get_dialog (MidoriSearchAction* search_action)
     column = gtk_tree_view_column_new ();
     renderer_pixbuf = gtk_cell_renderer_pixbuf_new ();
     gtk_tree_view_column_pack_start (column, renderer_pixbuf, FALSE);
+    g_object_set_data (G_OBJECT (treeview), "search-action", search_action);
     gtk_tree_view_column_set_cell_data_func (column, renderer_pixbuf,
         (GtkTreeCellDataFunc)midori_search_action_dialog_render_icon_cb,
         treeview, NULL);
@@ -1151,7 +1145,7 @@ midori_search_action_get_dialog (MidoriSearchAction* search_action)
     gtk_widget_set_sensitive (button, FALSE);
     gtk_box_pack_end (GTK_BOX (vbox), button, FALSE, FALSE, 0);
 
-    #ifdef HAVE_OSX
+    #if HAVE_OSX
     GtkWidget* icon;
     hbox = gtk_hbox_new (FALSE, 0);
     button = gtk_button_new ();
