@@ -18,14 +18,18 @@
 #include "midori-stock.h"
 
 #include "compat.h"
+#include "marshal.h"
 #include "sokoke.h"
-/* #include "gjs.h" */
 
 #include <string.h>
 #include <stdlib.h>
 #include <glib/gi18n.h>
 #include <webkit/webkit.h>
 
+#ifndef WEBKIT_CHECK_VERSION
+    #define WEBKIT_CHECK_VERSION(a,b,c) 0
+#endif
+
 /* This is unstable API, so we need to declare it */
 gchar*
 webkit_web_view_get_selected_text (WebKitWebView* web_view);
@@ -38,6 +42,7 @@ struct _MidoriView
 
     gchar* uri;
     gchar* title;
+    gchar* mime_type;
     GdkPixbuf* icon;
     gdouble progress;
     MidoriLoadStatus load_status;
@@ -53,6 +58,7 @@ struct _MidoriView
     gboolean middle_click_opens_selection;
     gboolean open_tabs_in_the_background;
     gboolean close_buttons_on_tabs;
+    MidoriNewPage open_new_pages_in;
 
     GtkWidget* menu_item;
     GtkWidget* tab_label;
@@ -89,12 +95,32 @@ midori_load_status_get_type (void)
     return type;
 }
 
+GType
+midori_new_view_get_type (void)
+{
+    static GType type = 0;
+    if (!type)
+    {
+        static const GEnumValue values[] = {
+         { MIDORI_NEW_VIEW_TAB, "MIDORI_NEW_VIEW_TAB", "New view in a tab" },
+         { MIDORI_NEW_VIEW_BACKGROUND, "MIDORI_NEW_VIEW_BACKGROUND",
+             "New view in a background tab" },
+         { MIDORI_NEW_VIEW_WINDOW, "MIDORI_NEW_VIEW_WINDOW",
+             "New view in a window" },
+         { 0, NULL, NULL }
+        };
+        type = g_enum_register_static ("MidoriNewView", values);
+    }
+    return type;
+}
+
 enum
 {
     PROP_0,
 
     PROP_URI,
     PROP_TITLE,
+    PROP_MIME_TYPE,
     PROP_ICON,
     PROP_LOAD_STATUS,
     PROP_PROGRESS,
@@ -112,6 +138,7 @@ enum {
     ATTACH_INSPECTOR,
     NEW_TAB,
     NEW_WINDOW,
+    NEW_VIEW,
     SEARCH_TEXT,
     ADD_BOOKMARK,
     SAVE_AS,
@@ -141,80 +168,6 @@ midori_view_settings_notify_cb (MidoriWebSettings* settings,
                                 GParamSpec*        pspec,
                                 MidoriView*        view);
 
-static void
-midori_cclosure_marshal_VOID__STRING_BOOLEAN (GClosure*     closure,
-                                              GValue*       return_value,
-                                              guint         n_param_values,
-                                              const GValue* param_values,
-                                              gpointer      invocation_hint,
-                                              gpointer      marshal_data)
-{
-    typedef void(*GMarshalFunc_VOID__STRING_BOOLEAN) (gpointer  data1,
-                                                      gpointer  arg_1,
-                                                      gboolean  arg_2,
-                                                      gpointer  data2);
-    register GMarshalFunc_VOID__STRING_BOOLEAN callback;
-    register GCClosure* cc = (GCClosure*) closure;
-    register gpointer data1, data2;
-
-    g_return_if_fail (n_param_values == 3);
-
-    if (G_CCLOSURE_SWAP_DATA (closure))
-    {
-        data1 = closure->data;
-        data2 = g_value_peek_pointer (param_values + 0);
-    }
-    else
-    {
-        data1 = g_value_peek_pointer (param_values + 0);
-        data2 = closure->data;
-    }
-    callback = (GMarshalFunc_VOID__STRING_BOOLEAN) (marshal_data
-        ? marshal_data : cc->callback);
-    callback (data1,
-              (gchar*)g_value_get_string (param_values + 1),
-              g_value_get_boolean (param_values + 2),
-              data2);
-}
-
-static void
-midori_cclosure_marshal_VOID__STRING_INT_STRING (GClosure*     closure,
-                                                 GValue*       return_value,
-                                                 guint         n_param_values,
-                                                 const GValue* param_values,
-                                                 gpointer      invocation_hint,
-                                                 gpointer      marshal_data)
-{
-    typedef void(*GMarshalFunc_VOID__STRING_INT_STRING) (gpointer  data1,
-                                                         gpointer  arg_1,
-                                                         gint      arg_2,
-                                                         gpointer  arg_3,
-                                                         gpointer  data2);
-    register GMarshalFunc_VOID__STRING_INT_STRING callback;
-    register GCClosure* cc = (GCClosure*) closure;
-    register gpointer data1, data2;
-
-    g_return_if_fail (n_param_values == 4);
-
-    if (G_CCLOSURE_SWAP_DATA (closure))
-    {
-        data1 = closure->data;
-        data2 = g_value_peek_pointer (param_values + 0);
-    }
-    else
-    {
-        data1 = g_value_peek_pointer (param_values + 0);
-        data2 = closure->data;
-    }
-    callback = (GMarshalFunc_VOID__STRING_INT_STRING) (marshal_data
-        ? marshal_data : cc->callback);
-    callback (data1,
-              (gchar*)g_value_get_string (param_values + 1),
-              g_value_get_int (param_values + 2),
-              (gchar*)g_value_get_string (param_values + 3),
-              data2);
-}
-
 static void
 midori_view_class_init (MidoriViewClass* class)
 {
@@ -290,6 +243,30 @@ midori_view_class_init (MidoriViewClass* class)
         G_TYPE_NONE, 1,
         G_TYPE_STRING);
 
+    /**
+     * MidoriView::new-view:
+     * @view: the object on which the signal is emitted
+     * @new_view: a newly created view
+     * @where: where to open the view
+     *
+     * Emitted when a new view is created. The value of
+     * @where determines where to open the view according
+     * to how it was opened and user preferences.
+     *
+     * Since: 0.1.2
+     */
+    signals[NEW_VIEW] = g_signal_new (
+        "new-view",
+        G_TYPE_FROM_CLASS (class),
+        (GSignalFlags)(G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION),
+        0,
+        0,
+        NULL,
+        midori_cclosure_marshal_VOID__OBJECT_ENUM,
+        G_TYPE_NONE, 2,
+        MIDORI_TYPE_VIEW,
+        MIDORI_TYPE_NEW_VIEW);
+
     signals[SEARCH_TEXT] = g_signal_new (
         "search-text",
         G_TYPE_FROM_CLASS (class),
@@ -348,6 +325,22 @@ midori_view_class_init (MidoriViewClass* class)
                                      NULL,
                                      G_PARAM_READWRITE));
 
+    /**
+    * MidoriView:mime-type:
+    *
+    * The MIME type of the currently loaded page.
+    *
+    * Since: 0.1.2
+    */
+    g_object_class_install_property (gobject_class,
+                                     PROP_MIME_TYPE,
+                                     g_param_spec_string (
+                                     "mime-type",
+                                     "MIME Type",
+                                     "The MIME type of the currently loaded page",
+                                     "text/html",
+                                     G_PARAM_READABLE));
+
     g_object_class_install_property (gobject_class,
                                      PROP_ICON,
                                      g_param_spec_object (
@@ -412,7 +405,7 @@ midori_view_class_init (MidoriViewClass* class)
                                      "Settings",
                                      "The associated settings",
                                      MIDORI_TYPE_WEB_SETTINGS,
-                                     G_PARAM_READWRITE));
+                                     flags));
 
     g_object_class_install_property (gobject_class,
                                      PROP_NET,
@@ -421,13 +414,68 @@ midori_view_class_init (MidoriViewClass* class)
                                      "Net",
                                      "The associated net",
                                      KATZE_TYPE_NET,
-                                     G_PARAM_READWRITE));
+                                     flags));
+}
+
+static GdkPixbuf*
+midori_view_mime_icon (GtkIconTheme* icon_theme,
+                       const gchar*  format,
+                       const gchar*  part1,
+                       const gchar*  part2,
+                       gchar**       name)
+{
+    GdkPixbuf* icon;
+
+    *name = part2 ? g_strdup_printf (format, part1, part2)
+        : g_strdup_printf (format, part1);
+    if (!(icon = gtk_icon_theme_load_icon (icon_theme, *name, 16, 0, NULL)))
+        g_free (*name);
+    return icon ? g_object_ref (icon) : NULL;
 }
 
 static void
-midori_view_notify_icon_cb (MidoriView* view,
-                            GParamSpec  pspec)
+midori_view_update_icon (MidoriView* view,
+                         GdkPixbuf*  icon)
 {
+    if (!icon)
+    {
+        GdkScreen* screen;
+        GtkIconTheme* icon_theme;
+        gchar** parts;
+        gchar* icon_name;
+
+        if ((screen = gtk_widget_get_screen (GTK_WIDGET (view))))
+        {
+            icon_theme = gtk_icon_theme_get_for_screen (screen);
+            if ((parts = g_strsplit (view->mime_type, "/", 2)))
+                parts = (parts[0] && parts[1]) ? parts : NULL;
+        }
+        else
+            parts = NULL;
+
+        if (parts)
+            icon = midori_view_mime_icon (icon_theme, "%s-%s",
+                                          parts[0], parts[1], &icon_name);
+        if (!icon && parts)
+            icon = midori_view_mime_icon (icon_theme, "gnome-mime-%s-%s",
+                                          parts[0], parts[1], &icon_name);
+        if (!icon && parts)
+            icon = midori_view_mime_icon (icon_theme, "%s-x-generic",
+                                          parts[0], NULL, &icon_name);
+        if (!icon && parts)
+            icon = midori_view_mime_icon (icon_theme, "gnome-mime-%s-x-generic",
+                                          parts[0], NULL, &icon_name);
+        if (view->item)
+            katze_item_set_icon (view->item, icon ? icon_name : NULL);
+        if (!icon)
+            icon = gtk_widget_render_icon (GTK_WIDGET (view),
+                GTK_STOCK_FILE, GTK_ICON_SIZE_MENU, NULL);
+    }
+    else if (view->item)
+        katze_item_set_icon (view->item, NULL);
+    katze_object_assign (view->icon, icon);
+    g_object_notify (G_OBJECT (view), "icon");
+
     if (view->tab_icon)
         katze_throbber_set_static_pixbuf (KATZE_THROBBER (view->tab_icon),
                                           view->icon);
@@ -441,26 +489,28 @@ static void
 midori_view_icon_cb (GdkPixbuf*  icon,
                      MidoriView* view)
 {
-    katze_object_assign (view->icon, icon);
-    g_object_notify (G_OBJECT (view), "icon");
+    midori_view_update_icon (view, icon);
 }
 
 static void
 _midori_web_view_load_icon (MidoriView* view)
 {
-    GdkPixbuf* pixbuf;
+    GdkPixbuf* pixbuf = katze_net_load_icon (view->net, view->uri,
+        (KatzeNetIconCb)midori_view_icon_cb, NULL, view);
 
-    pixbuf = katze_net_load_icon (view->net, view->uri,
-        (KatzeNetIconCb)midori_view_icon_cb, GTK_WIDGET (view), view);
-
-    katze_object_assign (view->icon, pixbuf);
-    g_object_notify (G_OBJECT (view), "icon");
+    midori_view_update_icon (view, pixbuf);
 }
 
 static void
-midori_view_notify_load_status_cb (MidoriView* view,
-                                   GParamSpec  pspec)
+midori_view_update_load_status (MidoriView*      view,
+                                MidoriLoadStatus load_status)
 {
+    if (view->load_status == load_status)
+        return;
+
+    view->load_status = load_status;
+    g_object_notify (G_OBJECT (view), "load-status");
+
     if (view->tab_icon)
         katze_throbber_set_animated (KATZE_THROBBER (view->tab_icon),
             view->load_status != MIDORI_LOAD_FINISHED);
@@ -474,11 +524,13 @@ webkit_web_view_load_started_cb (WebKitWebView*  web_view,
                                  WebKitWebFrame* web_frame,
                                  MidoriView*     view)
 {
-    view->load_status = MIDORI_LOAD_PROVISIONAL;
-    g_object_notify (G_OBJECT (view), "load-status");
+    g_object_freeze_notify (G_OBJECT (view));
 
+    midori_view_update_load_status (view, MIDORI_LOAD_PROVISIONAL);
     view->progress = 0.0;
     g_object_notify (G_OBJECT (view), "progress");
+
+    g_object_thaw_notify (G_OBJECT (view));
 }
 
 static void
@@ -487,20 +539,19 @@ webkit_web_view_load_committed_cb (WebKitWebView*  web_view,
                                    MidoriView*     view)
 {
     const gchar* uri;
-    GdkPixbuf* icon;
+
+    g_object_freeze_notify (G_OBJECT (view));
 
     uri = webkit_web_frame_get_uri (web_frame);
     katze_assign (view->uri, g_strdup (uri));
     g_object_notify (G_OBJECT (view), "uri");
     g_object_set (view, "title", NULL, NULL);
 
-    icon = gtk_widget_render_icon (GTK_WIDGET (view),
-        GTK_STOCK_FILE, GTK_ICON_SIZE_MENU, NULL);
-    katze_object_assign (view->icon, icon);
-    g_object_notify (G_OBJECT (view), "icon");
+    midori_view_update_icon (view, NULL);
 
-    view->load_status = MIDORI_LOAD_COMMITTED;
-    g_object_notify (G_OBJECT (view), "load-status");
+    midori_view_update_load_status (view, MIDORI_LOAD_COMMITTED);
+
+    g_object_thaw_notify (G_OBJECT (view));
 }
 
 static void
@@ -565,8 +616,7 @@ webkit_web_frame_load_done_cb (WebKitWebFrame* web_frame,
         g_free (data);
     }
 
-    view->load_status = MIDORI_LOAD_FINISHED;
-    g_object_notify (G_OBJECT (view), "load-status");
+    midori_view_update_load_status (view, MIDORI_LOAD_FINISHED);
 }
 
 static void
@@ -579,8 +629,13 @@ webkit_web_view_load_finished_cb (WebKitWebView*  web_view,
     GjsValue* document;
     GjsValue* links; */
 
+    g_object_freeze_notify (G_OBJECT (view));
+
     view->progress = 1.0;
     g_object_notify (G_OBJECT (view), "progress");
+    midori_view_update_load_status (view, MIDORI_LOAD_FINISHED);
+
+    g_object_thaw_notify (G_OBJECT (view));
 
     /* js_context = webkit_web_frame_get_global_context (web_frame);
     value = gjs_value_new (js_context, NULL);
@@ -642,7 +697,7 @@ gtk_widget_button_press_event_cb (WebKitWebView*  web_view,
     case 1:
         if (!link_uri)
             return FALSE;
-        #ifdef HAVE_OSX
+        #if HAVE_OSX
         /* FIXME: Test for Command key */
         if (0)
         #else
@@ -714,6 +769,12 @@ gtk_widget_button_press_event_cb (WebKitWebView*  web_view,
             }
         }
         break;
+    case 8:
+        midori_view_go_back (view);
+        return TRUE;
+    case 9:
+        midori_view_go_forward (view);
+        return TRUE;
     }
 
     return FALSE;
@@ -783,7 +844,7 @@ midori_web_view_menu_action_activate_cb (GtkWidget*  widget,
                                          MidoriView* view)
 {
     const gchar* action = g_object_get_data (G_OBJECT (widget), "action");
-    g_signal_emit_by_name (view, "activate-action", action);
+    g_signal_emit (view, signals[ACTIVATE_ACTION], 0, action);
 }
 
 static void
@@ -871,7 +932,7 @@ webkit_web_view_populate_popup_cb (WebKitWebView* web_view,
             && !strchr (view->selected_text, ' '))
         {
             menuitem = gtk_image_menu_item_new_with_mnemonic (
-                _("Open URL in New _Tab"));
+                _("Open Address in New _Tab"));
             icon = gtk_image_new_from_stock (GTK_STOCK_JUMP_TO,
                                              GTK_ICON_SIZE_MENU);
             gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menuitem), icon);
@@ -931,6 +992,90 @@ webkit_web_view_populate_popup_cb (WebKitWebView* web_view,
     }
 }
 
+static gboolean
+webkit_web_view_web_view_ready_cb (GtkWidget*  web_view,
+                                   MidoriView* view)
+{
+    GtkWidget* new_view = gtk_widget_get_parent (web_view);
+    MidoriNewView where = MIDORI_NEW_VIEW_TAB;
+    if (view->open_new_pages_in == MIDORI_NEW_PAGE_TAB)
+    {
+        if (view->open_tabs_in_the_background)
+            where = MIDORI_NEW_VIEW_BACKGROUND;
+    }
+    else if (view->open_new_pages_in == MIDORI_NEW_PAGE_WINDOW)
+        where = MIDORI_NEW_VIEW_WINDOW;
+
+    gtk_widget_show (new_view);
+    if (view->open_new_pages_in == MIDORI_NEW_PAGE_CURRENT)
+    {
+        g_debug ("Opening all pages in current tab not implemented");
+        g_signal_emit (view, signals[NEW_VIEW], 0, new_view, where);
+    }
+    else
+        g_signal_emit (view, signals[NEW_VIEW], 0, new_view, where);
+
+    return TRUE;
+}
+
+static GtkWidget*
+webkit_web_view_create_web_view_cb (GtkWidget*      web_view,
+                                    WebKitWebFrame* web_frame,
+                                    MidoriView*     view)
+{
+    GtkWidget* new_view = g_object_new (MIDORI_TYPE_VIEW,
+        "net", view->net,
+        "settings", view->settings,
+        NULL);
+    midori_view_set_uri (MIDORI_VIEW (new_view), "");
+    g_signal_connect (MIDORI_VIEW (new_view)->web_view, "web-view-ready",
+      G_CALLBACK (webkit_web_view_web_view_ready_cb), view);
+    return MIDORI_VIEW (new_view)->web_view;
+}
+
+static gboolean
+webkit_web_view_mime_type_decision_cb (GtkWidget*      web_view,
+                                       WebKitWebFrame* web_frame,
+                                       gpointer        request,
+                                       const gchar*    mime_type,
+                                       gpointer        decision,
+                                       MidoriView*     view)
+{
+    #if WEBKIT_CHECK_VERSION (1, 0, 0)
+    gchar* uri;
+    #endif
+
+    if (web_frame != webkit_web_view_get_main_frame (WEBKIT_WEB_VIEW (web_view)))
+        return FALSE;
+
+    katze_assign (view->mime_type, g_strdup (mime_type));
+    midori_view_update_icon (view, NULL);
+    g_object_notify (G_OBJECT (view), "mime-type");
+
+    /* TODO: Display contents with a Viewable if WebKit can't do it */
+    /* TODO: Offer downloading file if it cannot be displayed at all */
+
+    #if WEBKIT_CHECK_VERSION (1, 1, 0)
+    if (webkit_web_view_can_show_mime_type (WEBKIT_WEB_VIEW (web_view), mime_type))
+    #else
+    if (g_str_has_prefix (mime_type, "image/") ||
+        g_strrstr (mime_type, "script") ||
+        g_str_has_prefix (mime_type, "text/") || g_strrstr (mime_type, "xml"))
+    #endif
+        return TRUE;
+
+    #if WEBKIT_CHECK_VERSION (1, 0, 0)
+    uri = g_strdup_printf ("error:nodisplay %s",
+        webkit_network_request_get_uri (request));
+    midori_view_set_uri (view, uri);
+    g_free (uri);
+    #else
+    midori_view_set_uri (view, "error:nodisplay ");
+    #endif
+
+    return TRUE;
+}
+
 static void
 webkit_web_view_console_message_cb (GtkWidget*   web_view,
                                     const gchar* message,
@@ -938,7 +1083,7 @@ webkit_web_view_console_message_cb (GtkWidget*   web_view,
                                     const gchar* source_id,
                                     MidoriView*  view)
 {
-    g_signal_emit_by_name (view, "console-message", message, line, source_id);
+    g_signal_emit (view, signals[CONSOLE_MESSAGE], 0, message, line, source_id);
 }
 
 static void
@@ -964,6 +1109,7 @@ midori_view_init (MidoriView* view)
 {
     view->uri = NULL;
     view->title = NULL;
+    view->mime_type = g_strdup ("text/html");
     view->icon = gtk_widget_render_icon (GTK_WIDGET (view), GTK_STOCK_FILE,
                                          GTK_ICON_SIZE_MENU, NULL);
     view->progress = 0.0;
@@ -971,20 +1117,9 @@ midori_view_init (MidoriView* view)
     view->statusbar_text = NULL;
     view->link_uri = NULL;
     view->selected_text = NULL;
-    view->settings = midori_web_settings_new ();
     view->item = NULL;
 
     view->download_manager = NULL;
-
-    view->net = katze_net_new ();
-
-    g_object_connect (view,
-                      "signal::notify::icon",
-                      midori_view_notify_icon_cb, NULL,
-                      "signal::notify::load-status",
-                      midori_view_notify_load_status_cb, NULL,
-                      NULL);
-
     view->web_view = NULL;
 
     /* Adjustments are not created automatically */
@@ -998,8 +1133,9 @@ midori_view_finalize (GObject* object)
 
     view = MIDORI_VIEW (object);
 
-    g_signal_handlers_disconnect_by_func (view->settings,
-        midori_view_settings_notify_cb, view);
+    if (view->settings)
+        g_signal_handlers_disconnect_by_func (view->settings,
+            midori_view_settings_notify_cb, view);
 
     katze_assign (view->uri, NULL);
     katze_assign (view->title, NULL);
@@ -1052,7 +1188,7 @@ midori_view_set_property (GObject*      object,
         katze_assign (view->statusbar_text, g_value_dup_string (value));
         break;
     case PROP_SETTINGS:
-        midori_view_set_settings (view, g_value_dup_object (value));
+        midori_view_set_settings (view, g_value_get_object (value));
         break;
     case PROP_NET:
         katze_object_assign (view->net, g_value_dup_object (value));
@@ -1116,6 +1252,8 @@ midori_view_get_property (GObject*    object,
 GtkWidget*
 midori_view_new (KatzeNet* net)
 {
+    g_return_val_if_fail (!net || KATZE_IS_NET (net), NULL);
+
     return g_object_new (MIDORI_TYPE_VIEW, "net", net, NULL);
 }
 
@@ -1125,6 +1263,7 @@ _midori_view_update_settings (MidoriView* view)
     g_object_get (view->settings,
         "download-manager", &view->download_manager,
         "close-buttons-on-tabs", &view->close_buttons_on_tabs,
+        "open-new-pages-in", &view->open_new_pages_in,
         "middle-click-opens-selection", &view->middle_click_opens_selection,
         "open-tabs-in-the-background", &view->open_tabs_in_the_background,
         NULL);
@@ -1152,6 +1291,10 @@ midori_view_settings_notify_cb (MidoriWebSettings* settings,
         sokoke_widget_set_visible (view->tab_close,
                                    view->close_buttons_on_tabs);
     }
+    else if (name == g_intern_string ("open-new-pages-in"))
+    {
+        view->open_new_pages_in = g_value_get_enum (&value);
+    }
     else if (name == g_intern_string ("middle-click-opens-selection"))
     {
         view->middle_click_opens_selection = g_value_get_boolean (&value);
@@ -1175,15 +1318,26 @@ void
 midori_view_set_settings (MidoriView*        view,
                           MidoriWebSettings* settings)
 {
+    g_return_if_fail (MIDORI_IS_VIEW (view));
+    g_return_if_fail (!settings || MIDORI_IS_WEB_SETTINGS (settings));
+
+    if (view->settings == settings)
+        return;
+
     if (view->settings)
         g_signal_handlers_disconnect_by_func (view->settings,
             midori_view_settings_notify_cb, view);
-    katze_object_assign (view->settings, g_object_ref (settings));
-    if (view->web_view)
-        g_object_set (view->web_view, "settings", view->settings, NULL);
-    _midori_view_update_settings (view);
-    g_signal_connect (settings, "notify",
-        G_CALLBACK (midori_view_settings_notify_cb), view);
+
+    katze_object_assign (view->settings, settings);
+    if (settings)
+    {
+        g_object_ref (settings);
+        if (view->web_view)
+            g_object_set (view->web_view, "settings", settings, NULL);
+        _midori_view_update_settings (view);
+        g_signal_connect (settings, "notify",
+            G_CALLBACK (midori_view_settings_notify_cb), view);
+    }
     g_object_notify (G_OBJECT (view), "settings");
 }
 
@@ -1220,12 +1374,10 @@ midori_view_get_progress (MidoriView* view)
     return view->progress;
 }
 
-#ifdef WEBKIT_CHECK_VERSION
-#if WEBKIT_CHECK_VERSION (1, 0, 3)
 static WebKitWebView*
-webkit_web_inspector_inspect_web_view_cb (WebKitWebInspector* inspector,
-                                          WebKitWebView*      web_view,
-                                          MidoriView*         view)
+webkit_web_inspector_inspect_web_view_cb (gpointer       inspector,
+                                          WebKitWebView* web_view,
+                                          MidoriView*    view)
 {
     gchar* title;
     GtkWidget* window;
@@ -1282,18 +1434,12 @@ webkit_web_inspector_inspect_web_view_cb (WebKitWebInspector* inspector,
     g_signal_emit (view, signals[ATTACH_INSPECTOR], 0, inspector_view); */
     return WEBKIT_WEB_VIEW (inspector_view);
 }
-#endif
-#endif
 
 static void
 midori_view_construct_web_view (MidoriView* view)
 {
     WebKitWebFrame* web_frame;
-    #ifdef WEBKIT_CHECK_VERSION
-    #if WEBKIT_CHECK_VERSION (1, 0, 3)
-    WebKitWebInspector* inspector;
-    #endif
-    #endif
+    gpointer inspector;
 
     view->web_view = webkit_web_view_new ();
 
@@ -1327,6 +1473,16 @@ midori_view_construct_web_view (MidoriView* view)
                       "signal::destroy",
                       webkit_web_view_destroy_cb, web_frame,
                       NULL);
+    if (g_signal_lookup ("create-web-view", WEBKIT_TYPE_WEB_VIEW))
+        g_object_connect (view->web_view,
+                      "signal::create-web-view",
+                      webkit_web_view_create_web_view_cb, view,
+                      NULL);
+    if (g_signal_lookup ("mime-type-policy-decision-requested", WEBKIT_TYPE_WEB_VIEW))
+        g_object_connect (view->web_view,
+                      "signal::mime-type-policy-decision-requested",
+                      webkit_web_view_mime_type_decision_cb, view,
+                      NULL);
     g_object_connect (web_frame,
                       "signal::load-done",
                       webkit_web_frame_load_done_cb, view,
@@ -1337,15 +1493,14 @@ midori_view_construct_web_view (MidoriView* view)
     gtk_widget_show (view->web_view);
     gtk_container_add (GTK_CONTAINER (view), view->web_view);
 
-    #ifdef WEBKIT_CHECK_VERSION
-    #if WEBKIT_CHECK_VERSION (1, 0, 3)
-    inspector = webkit_web_view_get_inspector (WEBKIT_WEB_VIEW (view->web_view));
-    g_object_connect (inspector,
-                      "signal::inspect-web-view",
-                      webkit_web_inspector_inspect_web_view_cb, view,
-                      NULL);
-    #endif
-    #endif
+    if (katze_object_has_property (view->web_view, "web-inspector"))
+    {
+        inspector = katze_object_get_object (view->web_view, "web-inspector");
+        g_object_connect (inspector,
+                          "signal::inspect-web-view",
+                          webkit_web_inspector_inspect_web_view_cb, view,
+                          NULL);
+    }
 }
 
 /**
@@ -1366,12 +1521,14 @@ midori_view_set_uri (MidoriView*  view,
 
     g_return_if_fail (MIDORI_IS_VIEW (view));
 
+    if (!uri) uri = "";
+
     if (!view->web_view && view->uri
         && g_str_has_prefix (view->uri, "view-source:"))
     {
         g_signal_emit (view, signals[NEW_TAB], 0, uri);
     }
-    else if (!view->web_view && uri && g_str_has_prefix (uri, "view-source:"))
+    else if (!view->web_view && g_str_has_prefix (uri, "view-source:"))
     {
         katze_assign (view->uri, g_strdup (uri));
         g_object_notify (G_OBJECT (view), "uri");
@@ -1394,7 +1551,23 @@ midori_view_set_uri (MidoriView*  view,
         if (g_str_has_prefix (uri, "error:"))
         {
             data = NULL;
-            if (!strncmp (uri, "error:nodocs ", 13))
+            if (!strncmp (uri, "error:nodisplay ", 16))
+            {
+                gchar* title;
+
+                katze_assign (view->uri, g_strdup (&uri[16]));
+                title = g_strdup_printf (_("Document cannot be displayed"));
+                data = g_strdup_printf (
+                    "<html><head><title>%s</title></head>"
+                    "<body><h1>%s</h1>"
+                    "<img src=\"file://" DATADIR "/midori/logo-shade.png\" "
+                    "style=\"position: absolute; right: 15px; bottom: 15px;\">"
+                    "<p />The document %s cannot be displayed."
+                    "</body></html>",
+                    title, title, view->uri);
+                g_free (title);
+            }
+            else if (!strncmp (uri, "error:nodocs ", 13))
             {
                 gchar* title;
 
@@ -1422,6 +1595,10 @@ midori_view_set_uri (MidoriView*  view,
                 return;
             }
         }
+        else if (g_str_has_prefix (uri, "javascript:"))
+        {
+            midori_view_execute_script (view, &uri[11], NULL);
+        }
         else
         {
             katze_assign (view->uri, g_strdup (uri));
@@ -1757,8 +1934,9 @@ midori_view_tab_label_parent_set (GtkWidget*  tab_label,
 {
     GtkWidget* parent;
 
+    /* FIXME: Disconnect orientation notification
     if (old_parent)
-        /* FIXME: Disconnect orientation notification */;
+        ; */
 
     if (!(parent = gtk_widget_get_parent (tab_label)))
         return;
@@ -1992,8 +2170,15 @@ midori_view_can_zoom_out (MidoriView* view)
 gboolean
 midori_view_can_view_source (MidoriView* view)
 {
+    g_return_val_if_fail (MIDORI_IS_VIEW (view), FALSE);
+
     const gchar* uri = view->uri;
 
+    /* FIXME: Consider other types that are also text */
+    if (!g_str_has_prefix (view->mime_type, "text/")
+        && !g_strrstr (view->mime_type, "xml"))
+        return FALSE;
+
     #if HAVE_LIBSOUP
     if (g_str_has_prefix (uri, "http://") || g_str_has_prefix (uri, "https://"))
         return TRUE;
