@@ -16,6 +16,10 @@
 #include <glib/gi18n.h>
 #include <string.h>
 
+#if HAVE_CONFIG_H
+    #include <config.h>
+#endif
+
 struct _MidoriWebSettings
 {
     WebKitWebSettings parent_instance;
@@ -39,6 +43,7 @@ struct _MidoriWebSettings
 
     MidoriStartup load_on_startup;
     gchar* homepage;
+    gboolean show_crash_dialog;
     gchar* download_folder;
     gchar* download_manager;
     gchar* text_editor;
@@ -64,9 +69,16 @@ struct _MidoriWebSettings
     gboolean remember_last_downloaded_files;
 
     gchar* http_proxy;
+    MidoriIdentity identify_as;
+    gchar* ident_string;
     gint cache_size;
 };
 
+struct _MidoriWebSettingsClass
+{
+    WebKitWebSettingsClass parent_class;
+};
+
 G_DEFINE_TYPE (MidoriWebSettings, midori_web_settings, WEBKIT_TYPE_WEB_SETTINGS)
 
 enum
@@ -92,6 +104,7 @@ enum
 
     PROP_LOAD_ON_STARTUP,
     PROP_HOMEPAGE,
+    PROP_SHOW_CRASH_DIALOG,
     PROP_DOWNLOAD_FOLDER,
     PROP_DOWNLOAD_MANAGER,
     PROP_TEXT_EDITOR,
@@ -107,6 +120,8 @@ enum
     PROP_OPEN_TABS_NEXT_TO_CURRENT,
     PROP_OPEN_POPUPS_IN_TABS,
 
+    PROP_ENFORCE_96_DPI,
+    PROP_ENABLE_DEVELOPER_EXTRAS,
     PROP_ACCEPT_COOKIES,
     PROP_ORIGINAL_COOKIES_ONLY,
     PROP_MAXIMUM_COOKIE_AGE,
@@ -117,6 +132,8 @@ enum
     PROP_REMEMBER_LAST_DOWNLOADED_FILES,
 
     PROP_HTTP_PROXY,
+    PROP_IDENTIFY_AS,
+    PROP_IDENT_STRING,
     PROP_CACHE_SIZE
 };
 
@@ -149,7 +166,7 @@ midori_preferred_encoding_get_type (void)
          { MIDORI_ENCODING_RUSSIAN, "MIDORI_ENCODING_RUSSIAN", N_("Russian (KOI8-R)") },
          { MIDORI_ENCODING_UNICODE, "MIDORI_ENCODING_UNICODE", N_("Unicode (UTF-8)") },
          { MIDORI_ENCODING_WESTERN, "MIDORI_ENCODING_WESTERN", N_("Western (ISO-8859-1)") },
-         { MIDORI_ENCODING_WESTERN, "MIDORI_ENCODING_CUSTOM", N_("Custom...") },
+         { MIDORI_ENCODING_CUSTOM, "MIDORI_ENCODING_CUSTOM", N_("Custom...") },
          { 0, NULL, NULL }
         };
         type = g_enum_register_static ("MidoriPreferredEncoding", values);
@@ -210,6 +227,25 @@ midori_accept_cookies_get_type (void)
     return type;
 }
 
+GType
+midori_identity_get_type (void)
+{
+    static GType type = 0;
+    if (!type)
+    {
+        static const GEnumValue values[] = {
+         { MIDORI_IDENT_MIDORI, "MIDORI_IDENT_MIDORI", N_("Midori") },
+         { MIDORI_IDENT_SAFARI, "MIDORI_IDENT_SAFARI", N_("Safari") },
+         { MIDORI_IDENT_FIREFOX, "MIDORI_IDENT_FIREFOX", N_("Firefox") },
+         { MIDORI_IDENT_EXPLORER, "MIDORI_IDENT_EXPLORER", N_("Internet Explorer") },
+         { MIDORI_IDENT_CUSTOM, "MIDORI_IDENT_CUSTOM", N_("Custom...") },
+         { 0, NULL, NULL }
+        };
+        type = g_enum_register_static ("MidoriIdentity", values);
+    }
+    return type;
+}
+
 static void
 midori_web_settings_finalize (GObject* object);
 
@@ -388,6 +424,22 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      "http://www.google.com",
                                      flags));
 
+    /**
+    * MidoriWebSettings:show-crash-dialog:
+    *
+    * Show a dialog after Midori crashed.
+    *
+    * Since: 0.1.2
+    */
+    g_object_class_install_property (gobject_class,
+                                     PROP_SHOW_CRASH_DIALOG,
+                                     g_param_spec_boolean (
+                                     "show-crash-dialog",
+                                     _("Show crash dialog"),
+                                     _("Show a dialog after Midori crashed"),
+                                     TRUE,
+                                     flags));
+
     g_object_class_install_property (gobject_class,
                                      PROP_DOWNLOAD_FOLDER,
                                      g_param_spec_string (
@@ -453,6 +505,7 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      TRUE,
                                      flags));
 
+    g_type_class_ref (WEBKIT_TYPE_WEB_VIEW);
     g_object_class_install_property (gobject_class,
                                      PROP_OPEN_NEW_PAGES_IN,
                                      g_param_spec_enum (
@@ -461,7 +514,8 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      _("Where to open new pages"),
                                      MIDORI_TYPE_NEW_PAGE,
                                      MIDORI_NEW_PAGE_TAB,
-                                     G_PARAM_READABLE));
+    g_signal_lookup ("create-web-view", WEBKIT_TYPE_WEB_VIEW)
+        ? G_PARAM_READWRITE : G_PARAM_READABLE));
 
     g_object_class_install_property (gobject_class,
                                      PROP_OPEN_EXTERNAL_PAGES_IN,
@@ -478,7 +532,7 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      g_param_spec_boolean (
                                      "middle-click-opens-selection",
                                      _("Middle click opens Selection"),
-                                     _("Load an URL from the selection via middle click"),
+                                     _("Load an address from the selection via middle click"),
                                      FALSE,
                                      flags));
 
@@ -510,6 +564,39 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      G_PARAM_READABLE));
 
 
+    if (!g_object_class_find_property (gobject_class, "enforce-96-dpi"))
+    /**
+    * MidoriWebSettings:enforce-96-dpi:
+    *
+    * Whether to enforce a resolution of 96 DPI.
+    *
+    * Since: 0.1.2
+    */
+    g_object_class_install_property (gobject_class,
+                                     PROP_ENFORCE_96_DPI,
+                                     g_param_spec_boolean (
+                                     "enforce-96-dpi",
+                                     "Enforce 96 DPI",
+                                     "Whether to enforce a resolution of 96 DPI",
+                                     FALSE,
+                                     G_PARAM_READABLE));
+
+    if (!g_object_class_find_property (gobject_class, "enable-developer-extras"))
+    /**
+    * MidoriWebSettings:enable-developer-extras:
+    *
+    * Whether to enable extra developer tools.
+    *
+    * Since: 0.1.2
+    */
+    g_object_class_install_property (gobject_class,
+                                     PROP_ENABLE_DEVELOPER_EXTRAS,
+                                     g_param_spec_boolean (
+                                     "enable-developer-extras",
+                                     "Enable Developer Extras",
+                                     "Whether to enable extra developer tools",
+                                     FALSE,
+                                     G_PARAM_READABLE));
 
     g_object_class_install_property (gobject_class,
                                      PROP_ACCEPT_COOKIES,
@@ -519,7 +606,11 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      _("What type of cookies to accept"),
                                      MIDORI_TYPE_ACCEPT_COOKIES,
                                      MIDORI_ACCEPT_COOKIES_ALL,
+                                     #if HAVE_LIBSOUP
+                                     G_PARAM_READWRITE));
+                                     #else
                                      G_PARAM_READABLE));
+                                     #endif
 
     g_object_class_install_property (gobject_class,
                                      PROP_ORIGINAL_COOKIES_ONLY,
@@ -537,7 +628,7 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      _("Maximum cookie age"),
                                      _("The maximum number of days to save cookies for"),
                                      0, G_MAXINT, 30,
-                                     G_PARAM_READABLE));
+                                     G_PARAM_READWRITE));
 
 
 
@@ -583,10 +674,56 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      PROP_HTTP_PROXY,
                                      g_param_spec_string (
                                      "http-proxy",
-                                     _("HTTP Proxy"),
-                                     _("The proxy used for HTTP connections"),
+                                     _("Proxy Server"),
+                                     _("The proxy server used for HTTP connections"),
                                      NULL,
+                                     #if HAVE_LIBSOUP
+                                     G_PARAM_READWRITE));
+                                     #else
                                      G_PARAM_READABLE));
+                                     #endif
+
+    /**
+    * MidoriWebSettings:identify-as:
+    *
+    * What to identify as to web pages.
+    *
+    * Since: 0.1.2
+    */
+    g_object_class_install_property (gobject_class,
+                                     PROP_IDENTIFY_AS,
+                                     g_param_spec_enum (
+                                     "identify-as",
+        /* i18n: This refers to an application, not the 'user agent' string */
+                                     _("Identify as"),
+                                     _("What to identify as to web pages"),
+                                     MIDORI_TYPE_IDENTITY,
+                                     MIDORI_IDENT_MIDORI,
+                                     #if HAVE_LIBSOUP
+                                     G_PARAM_READWRITE));
+                                     #else
+                                     G_PARAM_READABLE));
+                                     #endif
+
+    /**
+    * MidoriWebSettings:ident-string:
+    *
+    * The browser identification string.
+    *
+    * Since: 0.1.2
+    */
+    g_object_class_install_property (gobject_class,
+                                     PROP_IDENT_STRING,
+                                     g_param_spec_string (
+                                     "ident-string",
+                                     _("Identification string"),
+                                     _("The application identification string"),
+                                     NULL,
+                                     #if HAVE_LIBSOUP
+                                     G_PARAM_READWRITE));
+                                     #else
+                                     G_PARAM_READABLE));
+                                     #endif
 
     g_object_class_install_property (gobject_class,
                                      PROP_CACHE_SIZE,
@@ -640,6 +777,72 @@ midori_web_settings_finalize (GObject* object)
     G_OBJECT_CLASS (midori_web_settings_parent_class)->finalize (object);
 }
 
+static gchar*
+generate_ident_string (MidoriIdentity identify_as)
+{
+    const gchar* platform =
+    #ifdef GDK_WINDOWING_X11
+    "X11";
+    #elif defined(GDK_WINDOWING_WIN32)
+    "Windows";
+    #elif defined(GDK_WINDOWING_QUARTZ)
+    "Macintosh";
+    #elif defined(GDK_WINDOWING_DIRECTFB)
+    "DirectFB";
+    #else
+    "Unknown";
+    #endif
+
+    const gchar* os =
+    #if HAVE_OSX
+    /* #if defined (HAVE_X86) */
+    "Intel Mac OS X";
+    /* #else
+    "PPC Mac OS X";
+    #endif */
+    #elif defined (G_OS_UNIX)
+    /* struct utsname name;
+    if (uname (&name) != -1)
+        String::format ("%s %s", name.sysname, name.machine);
+    else
+        "Unknown";*/
+    "Linux";
+    #elif defined (G_OS_WIN32)
+    // FIXME: Windows NT version
+    "Windows";
+    #else
+    "Unknown";
+    #endif
+
+    const gchar* appname = "Midori/"
+        G_STRINGIFY (MIDORI_MAJOR_VERSION) "."
+        G_STRINGIFY (MIDORI_MINOR_VERSION) "."
+        G_STRINGIFY (MIDORI_MICRO_VERSION);
+
+    const gchar* lang = pango_language_to_string (gtk_get_default_language ());
+
+    switch (identify_as)
+    {
+    case MIDORI_IDENT_MIDORI:
+        return g_strdup_printf ("%s (%s; %s; U; %s) WebKit/532+",
+                                appname, platform, os, lang);
+    case MIDORI_IDENT_SAFARI:
+        return g_strdup_printf ("Mozilla/5.0 (%s; U; %s; %s) "
+            "AppleWebKit/532+ (KHTML, like Gecko) Safari/419.3 %s",
+                                platform, os, lang, appname);
+    case MIDORI_IDENT_FIREFOX:
+        return g_strdup_printf ("Mozilla/5.0 (%s; U; %s; %s; rv:1.8.1) "
+            "Gecko/20061010 Firefox/2.0 %s",
+                                platform, os, lang, appname);
+    case MIDORI_IDENT_EXPLORER:
+        return g_strdup_printf ("Mozilla/4.0 (compatible; "
+            "MSIE 6.0; Windows NT 5.1; %s) %s",
+                                lang, appname);
+    default:
+        return g_strdup_printf ("%s", appname);
+    }
+}
+
 static void
 midori_web_settings_set_property (GObject*      object,
                                   guint         prop_id,
@@ -701,6 +904,9 @@ midori_web_settings_set_property (GObject*      object,
     case PROP_HOMEPAGE:
         katze_assign (web_settings->homepage, g_value_dup_string (value));
         break;
+    case PROP_SHOW_CRASH_DIALOG:
+        web_settings->show_crash_dialog = g_value_get_boolean (value);
+        break;
     case PROP_DOWNLOAD_FOLDER:
         katze_assign (web_settings->download_folder, g_value_dup_string (value));
         break;
@@ -788,6 +994,18 @@ midori_web_settings_set_property (GObject*      object,
     case PROP_HTTP_PROXY:
         katze_assign (web_settings->http_proxy, g_value_dup_string (value));
         break;
+    case PROP_IDENTIFY_AS:
+        web_settings->identify_as = g_value_get_enum (value);
+        if (web_settings->identify_as != MIDORI_IDENT_CUSTOM)
+        {
+            gchar* string = generate_ident_string (web_settings->identify_as);
+            g_object_set (object, "ident-string", string, NULL);
+            g_free (string);
+        }
+        break;
+    case PROP_IDENT_STRING:
+        katze_assign (web_settings->ident_string, g_value_dup_string (value));
+        break;
     case PROP_CACHE_SIZE:
         web_settings->cache_size = g_value_get_int (value);
         break;
@@ -858,6 +1076,9 @@ midori_web_settings_get_property (GObject*    object,
     case PROP_HOMEPAGE:
         g_value_set_string (value, web_settings->homepage);
         break;
+    case PROP_SHOW_CRASH_DIALOG:
+        g_value_set_boolean (value, web_settings->show_crash_dialog);
+        break;
     case PROP_DOWNLOAD_FOLDER:
         g_value_set_string (value, web_settings->download_folder);
         break;
@@ -899,6 +1120,12 @@ midori_web_settings_get_property (GObject*    object,
         g_value_set_boolean (value, web_settings->open_popups_in_tabs);
         break;
 
+    case PROP_ENFORCE_96_DPI:
+        g_value_set_boolean (value, FALSE);
+        break;
+    case PROP_ENABLE_DEVELOPER_EXTRAS:
+        g_value_set_boolean (value, FALSE);
+        break;
     case PROP_ACCEPT_COOKIES:
         g_value_set_enum (value, web_settings->accept_cookies);
         break;
@@ -925,6 +1152,12 @@ midori_web_settings_get_property (GObject*    object,
     case PROP_HTTP_PROXY:
         g_value_set_string (value, web_settings->http_proxy);
         break;
+    case PROP_IDENTIFY_AS:
+        g_value_set_enum (value, web_settings->identify_as);
+        break;
+    case PROP_IDENT_STRING:
+        g_value_set_string (value, web_settings->ident_string);
+        break;
     case PROP_CACHE_SIZE:
         g_value_set_int (value, web_settings->cache_size);
         break;
