@@ -12,10 +12,8 @@
 #ifndef __MIDORI_H__
 #define __MIDORI_H__
 
-#include "midori-addons.h"
 #include "midori-app.h"
 #include "midori-browser.h"
-#include "midori-console.h"
 #include "midori-extension.h"
 #include "midori-locationaction.h"
 #include "midori-locationentry.h"
