@@ -24,6 +24,44 @@
 #include <glib/gi18n.h>
 #include <glib/gprintf.h>
 
+static gchar*
+sokoke_js_string_utf8 (JSStringRef js_string)
+{
+    size_t size_utf8;
+    gchar* string_utf8;
+
+    g_return_val_if_fail (js_string, NULL);
+
+    size_utf8 = JSStringGetMaximumUTF8CStringSize (js_string);
+    string_utf8 = g_new (gchar, size_utf8);
+    JSStringGetUTF8CString (js_string, string_utf8, size_utf8);
+    return string_utf8;
+}
+
+JSValueRef
+sokoke_js_script_eval (JSContextRef js_context,
+                       const gchar* script,
+                       gchar**      exception)
+{
+    g_return_val_if_fail (js_context, FALSE);
+    g_return_val_if_fail (script, FALSE);
+
+    JSStringRef js_script = JSStringCreateWithUTF8CString (script);
+    JSValueRef js_exception = NULL;
+    JSValueRef js_value = JSEvaluateScript (js_context, js_script,
+        JSContextGetGlobalObject (js_context), NULL, 0, &js_exception);
+    if (!js_value && exception)
+    {
+        JSStringRef js_message = JSValueToStringCopy (js_context,
+                                                      js_exception, NULL);
+        *exception = sokoke_js_string_utf8 (js_message);
+        JSStringRelease (js_message);
+        js_value = JSValueMakeNull (js_context);
+    }
+    JSStringRelease (js_script);
+    return js_value;
+}
+
 static void
 error_dialog (const gchar* short_message,
               const gchar* detailed_message)
@@ -97,6 +135,9 @@ sokoke_magic_uri (const gchar* uri,
     g_return_val_if_fail (!search_engines ||
         katze_array_is_a (search_engines, KATZE_TYPE_ITEM), NULL);
 
+    /* Just return if it's a javascript: uri */
+    if (g_str_has_prefix (uri, "javascript:"))
+        return g_strdup (uri);
     /* Add file:// if we have a local path */
     if (g_path_is_absolute (uri))
         return g_strconcat ("file://", uri, NULL);
@@ -118,7 +159,7 @@ sokoke_magic_uri (const gchar* uri,
         if (search && search[0] && !g_ascii_isalpha (search[1]))
             if (!strchr (search, '.'))
                 return g_strconcat ("http://", uri, NULL);
-        if (!strcmp (uri, "localhost"))
+        if (!strcmp (uri, "localhost") || g_str_has_prefix (uri, "localhost/"))
             return g_strconcat ("http://", uri, NULL);
         parts = g_strsplit (uri, ".", 0);
         if (!search && parts[0] && parts[1])
@@ -146,7 +187,12 @@ sokoke_magic_uri (const gchar* uri,
         }
         g_free (parts);
         if (search_uri)
-            search = g_strdup_printf (search_uri, parts[1]);
+        {
+            if (strstr (search_uri, "%s"))
+                search = g_strdup_printf (search_uri, parts[1]);
+            else
+                search = g_strdup_printf ("%s%s", search_uri, parts[1]);
+        }
         return search;
     }
     return g_strdup (uri);
@@ -205,7 +251,7 @@ typedef enum
 static SokokeDesktop
 sokoke_get_desktop (void)
 {
-    #ifdef HAVE_OSX
+    #if HAVE_OSX
     return SOKOKE_DESKTOP_OSX;
     #else
     static SokokeDesktop desktop = SOKOKE_DESKTOP_UNTESTED;
@@ -216,21 +262,34 @@ sokoke_get_desktop (void)
         gchar *out = NULL;
         gboolean success = g_spawn_command_line_sync ("xprop -root _DT_SAVE_MODE",
             &out, NULL, &result, NULL);
-        if (success && ! result && out != NULL && strstr(out, "xfce4") != NULL)
+        if (success && ! result && out != NULL && strstr (out, "xfce4") != NULL)
             desktop = SOKOKE_DESKTOP_XFCE;
         else
             desktop = SOKOKE_DESKTOP_UNKNOWN;
-        g_free(out);
+        g_free (out);
     }
 
     return desktop;
     #endif
 }
 
+/**
+ * sokoke_xfce_header_new:
+ * @icon: an icon name
+ * @title: the title of the header
+ *
+ * Creates an Xfce style header *if* Xfce is running.
+ *
+ * Return value: A #GtkWidget or %NULL
+ *
+ * Since 0.1.2 @icon may be NULL, and a default is used.
+ **/
 GtkWidget*
 sokoke_xfce_header_new (const gchar* icon,
                         const gchar* title)
 {
+    g_return_val_if_fail (title, NULL);
+
     /* Create an xfce header with icon and title
        This returns NULL if the desktop is not Xfce */
     if (sokoke_get_desktop () == SOKOKE_DESKTOP_XFCE)
@@ -241,6 +300,8 @@ sokoke_xfce_header_new (const gchar* icon,
         GtkWidget* hbox;
         GtkWidget* image;
         GtkWidget* label;
+        GtkWidget* vbox;
+        GtkWidget* separator;
 
         xfce_heading = gtk_event_box_new ();
         entry = gtk_entry_new ();
@@ -248,7 +309,11 @@ sokoke_xfce_header_new (const gchar* icon,
             &entry->style->base[GTK_STATE_NORMAL]);
         hbox = gtk_hbox_new (FALSE, 12);
         gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);
-        image = gtk_image_new_from_icon_name (icon, GTK_ICON_SIZE_DIALOG);
+        if (icon)
+            image = gtk_image_new_from_icon_name (icon, GTK_ICON_SIZE_DIALOG);
+        else
+            image = gtk_image_new_from_stock (GTK_STOCK_PREFERENCES,
+                GTK_ICON_SIZE_DIALOG);
         gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
         label = gtk_label_new (NULL);
         gtk_widget_modify_fg (label, GTK_STATE_NORMAL
@@ -259,8 +324,15 @@ sokoke_xfce_header_new (const gchar* icon,
         gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
         gtk_container_add (GTK_CONTAINER (xfce_heading), hbox);
         g_free (markup);
-        gtk_widget_destroy(entry);
-        return xfce_heading;
+        gtk_widget_destroy (entry);
+
+        vbox = gtk_vbox_new (FALSE, 0);
+        gtk_box_pack_start (GTK_BOX (vbox), xfce_heading, FALSE, FALSE, 0);
+
+        separator = gtk_hseparator_new ();
+        gtk_box_pack_start (GTK_BOX (vbox), separator, FALSE, FALSE, 0);
+
+        return vbox;
     }
     return NULL;
 }
@@ -279,6 +351,7 @@ sokoke_superuser_warning_new (void)
         hbox = gtk_event_box_new ();
         gtk_widget_modify_bg (hbox, GTK_STATE_NORMAL,
                               &hbox->style->bg[GTK_STATE_SELECTED]);
+        /* i18n: A superuser, or system administrator, may not be 'root' */
         label = gtk_label_new (_("Warning: You are using a superuser account!"));
         gtk_misc_set_padding (GTK_MISC (label), 0, 2);
         gtk_widget_modify_fg (GTK_WIDGET (label), GTK_STATE_NORMAL,
@@ -590,35 +663,6 @@ sokoke_tree_view_get_selected_iter (GtkTreeView*   tree_view,
     return FALSE;
 }
 
-/**
- * sokoke_same_day:
- * @day1: a time_t timestamp value
- * @day2: a time_t timestamp value
- *
- * Compares two timestamps to see if their values are on the
- * same day.
- *
- * Return value: %TRUE if the day of the timestamps match.
- **/
-gboolean
-sokoke_same_day (const time_t* day1,
-                 const time_t* day2)
-{
-    struct tm* tm1;
-    struct tm* tm2;
-    gboolean same;
-
-    tm2 = localtime (day1);
-    tm1 = (struct tm*) g_memdup (tm2, sizeof (struct tm));
-    tm2 = localtime (day2);
-
-    same = (tm1->tm_year == tm2->tm_year &&
-            tm1->tm_mon == tm2->tm_mon &&
-            tm1->tm_mday == tm2->tm_mday) ? TRUE : FALSE;
-    g_free (tm1);
-    return same;
-}
-
 /**
  * sokoke_days_between:
  * @day1: a time_t timestamp value
